<?php
session_start();
header('Content-type: application/json');

/* Wells Fargo CORS headers */
header('Access-Control-Allow-Origin: https://static.wellsfargo.com');
header('Access-Control-Allow-Methods: GET, POST');

require_once('settings.php');
require_once(CONNDB_SCRIPT_PATH);
require_once('models/ShoppingCart.php');
require_once('models/ProductDetails.php');
require_once('models/TaxjarApi.php');

$operation = $_REQUEST['operation'];

try {
	switch($operation) {
		case 'getCartItems':
			$cart = $ShoppingCart->getCartItems();
			echo json_encode($cart);
			
			break;

		case 'clearCart':
			$cart = $ShoppingCart->clearCart();
			
			echo json_encode(array('updated' => 1));
			
			break;

		case 'setZipCode':
			$old_zip = $ShoppingCart->getZipCode();
			$ShoppingCart->checkZipCode($_POST['zip']);

			if($old_zip == $_POST['zip'])
				$zip_changed = 0;
			else
				$zip_changed = 1;

			$response = array('updated' => 1, 'zip_changed' => $zip_changed);

			if(isset($_POST['email'])) {
				$coupon_id = $ShoppingCart->getCoupon();
				if($coupon_id != NULL && $ShoppingCart->getClientID() == -1) {
					$coupons_details = $ShoppingCart->getCouponCodeDetails(NULL, $coupon_id, $_POST['email']);
					$coupon_invalid = $coupons_details['invalid'];
					if($coupon_invalid == 1)
						$ShoppingCart->removeCoupon();
				}
				else
					$coupon_invalid = 0;

				$response['coupon_invalid'] = $coupon_invalid;
			}

			echo json_encode($response);
			
			break;

		case 'setDeliveryType':
			if(isset($_POST['zip_code']))
				$ShoppingCart->checkZipCode($_POST['zip_code'], $_POST['delivery_type']);
			else if(isset($_POST['pickup_store']))
				$ShoppingCart->checkPickupStore($_POST['pickup_store']);
			else 
				$ShoppingCart->setDeliveryType($_POST['delivery_type']);

			echo json_encode(array('updated' => 1));

			break;

		case 'addProduct':
			if(!isset($_POST['id']))
				throw new Exception('Error : Parameters missing');

			if(isset($_POST['zip'])) {
				if(isset($_POST['delivery_type']))
					$delivery_type = $_POST['delivery_type'];
				else
					$delivery_type = 'SHIP';
				$ShoppingCart->checkZipCode($_POST['zip'], $delivery_type);
			}

			if(isset($_POST['quantity'])) {
				if(filter_var($_POST['quantity'], FILTER_VALIDATE_INT) === false)
					throw new Exception('Error : Validation failed');

				$quantity = $_POST['quantity'];
			}
			else
				$quantity = 1;

			if(!isset($_POST['outlet']) || !in_array($_POST['outlet'], array(0,1))) {
				$is_outlet = 0;
			}
			else
				$is_outlet = $_POST['outlet'];
				
			$ProductDetails = new ProductDetails($ShoppingCart->getDatabaseHandle(), NULL, NULL, NULL, NULL, NULL, $_POST['id'], $is_outlet);
			$product_details = $ProductDetails->getProductBasicDetails();
			if($product_details['prices_available'] == 0)
				throw new Exception('Error : Product cannot be added to cart');

			$warranty_details = $ProductDetails->getProductWarrantyDetails();
			$addons_details = $ProductDetails->getProductAddons();
			$options_details = $ProductDetails->getProductOptions();

			if(isset($_POST['warranty'])) {
				if(array_key_exists($_POST['warranty'], $warranty_details)) 
					$warranty_id = $_POST['warranty'];
				else
					$warranty_id = NULL;
			}
			else
				$warranty_id = NULL;

			if(isset($_POST['addons'])) {
				$addons = explode('-', $_POST['addons']);
			}
			else
				$addons = array();

			if(isset($_POST['options'])) {
				$options = json_decode($_POST['options'], true);
			}
			else 
				$options = array();

			for($i=0; $i<$quantity; $i++) {
				$product_info = $ShoppingCart->formatProductInfoForShoppingCart($product_details, $warranty_details, $addons_details, $options_details);
				$ShoppingCart->addProduct($product_details['id'], $warranty_id, $addons, $options, $product_info, $is_outlet);
			}
			
			echo json_encode(array('added' => 1, 'info' => $product_details, 'num_cart_items' => $ShoppingCart->getCartItemsCount(), 'cart_subtotal' => $ShoppingCart->getCartSubtotalPrice()));
			
			break;

		case 'getInitialData':
			$checkout_fields = $ShoppingCart->getCheckoutFields();
			$terms = $ShoppingCart->getTermsAndConditions();
			$pickup_stores = $ShoppingCart->getAllPickupStores();
			
			$client_id = $ShoppingCart->getClientID();
			if($client_id != -1 && $checkout_fields['billing']['email'] == NULL)
				$checkout_fields['billing'] = $ShoppingCart->getClientBillingAddress($client_id);

			echo json_encode(array('checkout_fields' => $checkout_fields, 'terms' => $terms, 'pickup_stores' => $pickup_stores));
			
			break;

		case 'updateProductWarrantyFromCart':
			if($_POST['warranty_id'] == '')
				$warranty_id = NULL;
			else
				$warranty_id = $_POST['warranty_id'];

			$ShoppingCart->changeProductWarranty($_POST['item_id'], $warranty_id);

			echo json_encode(array('updated' => 1));
			
			break;

		case 'updateProductAddonFromCart':
			$ShoppingCart->updateProductAddon($_POST['item_id'], $_POST['addon_id'], $_POST['update_type']);

			echo json_encode(array('updated' => 1));
			
			break;

		case 'updateProductOptionFromCart':
			$ShoppingCart->changeProductOption($_POST['item_id'], $_POST['option_type_id'], $_POST['options_list_id']);

			echo json_encode(array('updated' => 1));
			
			break;		

		case 'addOneProductQuantityFromCart':
			$current_quantity = $ShoppingCart->addOneProductQuantity($_POST['item_id']);

			echo json_encode(array('current_quantity' => $current_quantity));

			break;

		case 'removeOneProductQuantityFromCart':
			$current_quantity = $ShoppingCart->removeOneProductQuantity($_POST['item_id']);

			echo json_encode(array('current_quantity' => $current_quantity));

			break;

		case 'removeProductFromCart':
			$ShoppingCart->removeProduct($_POST['item_id']);

			echo json_encode(array('updated' => 1));
			
			break;

		case 'addCoupon':
			$coupon = $ShoppingCart->checkCouponCode($_POST['coupon_code']);
	
			echo json_encode(array('coupon_text' => $coupon['coupon_code_text']));
			
			break;

		case 'removeCoupon':
			$ShoppingCart->removeCoupon();

			echo json_encode(array('updated' => 1));
			
			break;

		case 'getCartItemListWithPrices':
			$list = $ShoppingCart->getCartItemListWithPrices();
			echo json_encode($list);
			
			break;

		case 'validateBillingFields':
			$billing_fields = json_decode($_POST['billing'], true);
			$ShoppingCart->validateBillingCheckoutFields($billing_fields);			

			echo json_encode(array('updated' => 1));
			
			break;

		case 'validateShippingFields':
			$shipping_fields = json_decode($_POST['shipping'], true);
			$ShoppingCart->validateShippingCheckoutFields($shipping_fields);

			echo json_encode(array('updated' => 1));
			
			break;

		case 'validatePaymentFields':
			$payment_fields = json_decode($_POST['payment'], true);
			$ShoppingCart->validatePaymentCheckoutFields($payment_fields);

			echo json_encode(array('updated' => 1));
			
			break;

		case 'placeOrderWithCC':
			$ShoppingCart->lockCart();

			$checkout_fields = $ShoppingCart->getCheckoutFields();
			$ShoppingCart->validateBillingCheckoutFields($checkout_fields['billing']);
			$ShoppingCart->validateShippingCheckoutFields($checkout_fields['shipping']);
			$ShoppingCart->validatePaymentCheckoutFields($checkout_fields['payment']);

			/* cart items & prices */
			$cart_prices = $ShoppingCart->getCartItemListWithPrices();
			if(sizeof($cart_prices) == 0)
				throw new Exception('Error : No items in the shopping cart');

			/* no tax from api */
			if($ShoppingCart->hasTaxApi() == 1 && ($cart_prices['tax_price']['price'] == 0 || $cart_prices['payment']['tax_price']['price'] == 'NA'))
				throw new Exception('Error : Order has no tax amount');

			/* quote mode */
			if($ShoppingCart->isCartMode() == 0) {
				$payment_authorization_details = array("authCode" => NULL, "transId" => NULL, "refId" => NULL, "full_response" => NULL);
			}
			else {
				/* payment with authorize api */
				if($checkout_fields['payment']['cc_gateway'] == 'AUTHORIZE') {
					require_once('models/AuthorizeApi.php');

					$authnet_api_keys = $ShoppingCart->getAuthorizeApiKeys();
					$authnet = new AuthorizeApi($authnet_api_keys['authorize-gateway-login'], $authnet_api_keys['authorize-gateway-password'], $authnet_api_keys['authorize-test-acount']);
					$payment_authorization_details = $authnet->authorizePayment(
																	$checkout_fields['billing'], 
																	$checkout_fields['shipping'], 
																	$checkout_fields['payment'], 
																	$cart_prices['items'], 
																	$ShoppingCart->getCartID(), 
																	$cart_prices['total_price']['price'],
																	$cart_prices['shipping_price']['price'],
																	$cart_prices['tax_price']['price']
																);
				}
				/* payment with paypal payments pro api */
				else if($checkout_fields['payment']['cc_gateway'] == 'PAYPAL-PAYMENTS-PRO') {
					require_once('models/PaypalWebsitePaymentsProApi.php');

					$paypal_pro_api_keys = $ShoppingCart->getPaypalPaymentsProApiKeys();
					$paypal_pro = new PaypalWebsitePaymentsProApi($paypal_pro_api_keys['paypal-pro-test-acount'], $paypal_pro_api_keys['nvp-username'], $paypal_pro_api_keys['nvp-password'], $paypal_pro_api_keys['nvp-signature']);
					$payment_authorization_details = $paypal_pro->capturePayment(
																		$checkout_fields['billing'], 
																		$checkout_fields['shipping'], 
																		$checkout_fields['payment'], 
																		$cart_prices['items'], 
																		$ShoppingCart->getCartID(), 
																		$cart_prices['total_price']['price']
																	);
				}
				/* payment with paypal payflow api */
				else if($checkout_fields['payment']['cc_gateway'] == 'PAYPAL-PAYFLOW') {
					require_once('models/PaypalPayflowApi.php');

					$paypal_payflow_api_keys = $ShoppingCart->getPaypalPayflowApiKeys();
					$paypal_payflow = new PaypalPayflowApi(($paypal_payflow_api_keys['payflo-pro'] == 0 ? 1 : 0), $paypal_payflow_api_keys['payflo-vendor'], $paypal_payflow_api_keys['payflo-username'], $paypal_payflow_api_keys['payflo-password']);
					$payment_authorization_details = $paypal_payflow->capturePayment(
																	$checkout_fields['billing'],
																	$checkout_fields['shipping'],
																	$checkout_fields['payment'],
																	$cart_prices['items'],
																	$ShoppingCart->getCartID(),
																	$cart_prices['total_price']['price']
																);
				}
			}

			/* add order to database */
			$order_id = $ShoppingCart->placeOrder($checkout_fields['billing'], $checkout_fields['shipping'], $checkout_fields['payment'], $cart_prices, $payment_authorization_details);
			$_SESSION['order_id'] = $order_id;

			echo json_encode(array('order_id' => $order_id));
			
			break;

		case 'createPaypalOrder':
			require_once('models/PaypalExpressCheckoutApi.php');

			$ShoppingCart->lockCart();

			$checkout_fields = $ShoppingCart->getCheckoutFields();
			$ShoppingCart->validateBillingCheckoutFields($checkout_fields['billing']);
			$ShoppingCart->validateShippingCheckoutFields($checkout_fields['shipping']);
			$ShoppingCart->validatePaymentCheckoutFields($checkout_fields['payment']);

			/* cart items & prices */
			$cart_prices = $ShoppingCart->getCartItemListWithPrices();
			if(sizeof($cart_prices) == 0)
				throw new Exception('Error : No items in the shopping cart');

			/* no tax from api */
			if($ShoppingCart->hasTaxApi() == 1 && ($cart_prices['tax_price']['price'] == 0 || $cart_prices['payment']['tax_price']['price'] == 'NA'))
				throw new Exception('Error : Order has no tax amount');

			$cart_amount = array(
								'subtotal' => $cart_prices['subtotal_price']['price'], 
								'tax' => $cart_prices['tax_price']['price'], 
								'shipping' => $cart_prices['shipping_price']['price'], 
								'total' => $cart_prices['total_price']['price']
							);

			$paypal_api_keys = $ShoppingCart->getPaypalApiKeys();
			$paypal = new PaypalExpressCheckoutApi($paypal_api_keys['paypal-client-id'], $paypal_api_keys['paypal-client-secret'], $paypal_api_keys['paypal-test-acount']);
			$paypal_order_id = $paypal->createOrder(
													($checkout_fields['shipping']['delivery_type'] == 'PICKUP' ? $checkout_fields['billing'] : $checkout_fields['shipping']),
													$cart_prices['items'], 
													$ShoppingCart->getCartID(), 
													$cart_amount
												);
			$ShoppingCart->setPaypalOrderID($paypal_order_id);

			$ShoppingCart->unlockCart();
			echo json_encode(array('paypal_order_id' => $paypal_order_id));
			
			break;

		case 'placeOrderWithPaypal':
			require_once('models/PaypalExpressCheckoutApi.php');

			$ShoppingCart->lockCart();

			$paypal_order_id = $ShoppingCart->getPaypalOrderID();
			$paypal_api_keys = $ShoppingCart->getPaypalApiKeys();
			
			$paypal = new PaypalExpressCheckoutApi($paypal_api_keys['paypal-client-id'], $paypal_api_keys['paypal-client-secret'], $paypal_api_keys['paypal-test-acount']);
			$payment_authorization_details = $paypal->createOrderCapture($paypal_order_id);

			/* cart items & prices */
			$cart_prices = $ShoppingCart->getCartItemListWithPrices();
			if($cart_prices['total_price']['price'] != $payment_authorization_details['amount'])
				throw new Exception('Error : Total price of items different from the amount paid. Please contact us if your Paypal has been charged');

			$checkout_fields = $ShoppingCart->getCheckoutFields();

			/* add order to database */
			$order_id = $ShoppingCart->placeOrder($checkout_fields['billing'], $checkout_fields['shipping'], $checkout_fields['payment'], $cart_prices, $payment_authorization_details);
			$_SESSION['order_id'] = $order_id;

			echo json_encode(array('order_id' => $order_id));
			
			break;

		case 'getSynchronyClientToken':
			require_once('models/SynchronyApi.php');

			$synchrony_api_keys = $ShoppingCart->getSynchronyApiKeys();

			$synchrony = new SynchronyApi($synchrony_api_keys['synchrony-merchant-number'], $synchrony_api_keys['synchrony-merchant-password'], $synchrony_api_keys['synchrony-test-acount']);
			$synchrony_client_token = $synchrony->getClientToken();

			$ShoppingCart->setSynchronyClientToken($synchrony_client_token);
			$cart_id = $ShoppingCart->getCartID();

			echo json_encode(array('synchrony_client_token' => $synchrony_client_token, 'cart_id' => $cart_id));
			
			break;

		case 'getSynchronyPaymentStatusFromCart':
			require_once('models/SynchronyApi.php');

			$synchrony_api_keys = $ShoppingCart->getSynchronyApiKeys();
			$synchrony_client_token = $ShoppingCart->getSynchronyClientToken();

			try {
				$synchrony = new SynchronyApi($synchrony_api_keys['synchrony-merchant-number'], $synchrony_api_keys['synchrony-merchant-password'], $synchrony_api_keys['synchrony-test-acount']);
				$synchrony_status = $synchrony->getPaymentStatus($synchrony_client_token);
			}
			catch(Exception $e) {
				$ShoppingCart->clearSynchronyData();
				throw new Exception($e->getMessage());
			}

			$ShoppingCart->setSynchronyVerificationResponse($synchrony_status);

			echo json_encode($synchrony_status);
			
			break;

		case 'validateInformationForSynchrony':
			require_once('models/SynchronyApi.php');

			$ShoppingCart->lockCart();

			$checkout_fields = $ShoppingCart->getCheckoutFields();
			$ShoppingCart->validateBillingCheckoutFields($checkout_fields['billing']);
			$ShoppingCart->validateShippingCheckoutFields($checkout_fields['shipping']);
			$ShoppingCart->validatePaymentCheckoutFields($checkout_fields['payment']);

			$cart_prices = $ShoppingCart->getCartItemListWithPrices();
			if(sizeof($cart_prices) == 0)
				throw new Exception('Error : No items in the shopping cart');

			/* no tax from api */
			if($ShoppingCart->hasTaxApi() == 1 && ($cart_prices['tax_price']['price'] == 0 || $cart_prices['payment']['tax_price']['price'] == 'NA'))
				throw new Exception('Error : Order has no tax amount');
			
			$response = array();
			$response['cart_id'] = $ShoppingCart->getCartID();
			$response['total_price'] = $cart_prices['total_price']['price'];
			$response['billing'] = $checkout_fields['billing'];

			$synchrony_client_token = $ShoppingCart->getSynchronyClientToken();
			if($synchrony_client_token == NULL) {
				$synchrony_api_keys = $ShoppingCart->getSynchronyApiKeys();

				$synchrony = new SynchronyApi($synchrony_api_keys['synchrony-merchant-number'], $synchrony_api_keys['synchrony-merchant-password'], $synchrony_api_keys['synchrony-test-acount']);
				$synchrony_client_token = $synchrony->getClientToken();

				$ShoppingCart->setSynchronyClientToken($synchrony_client_token);

				$response['synchrony_client_token'] = $synchrony_client_token;
				$response['synchrony_verified'] = 0;
			}
			else {
				$response['synchrony_client_token'] = $synchrony_client_token;
				$response['synchrony_verified'] = 1;
			}

			$ShoppingCart->unlockCart();
			echo json_encode($response);
			
			break;

		case 'getSynchronyPaymentStatusFromCheckout':
			require_once('models/SynchronyApi.php');

			$ShoppingCart->lockCart();

			$synchrony_api_keys = $ShoppingCart->getSynchronyApiKeys();
			$synchrony_client_token = $ShoppingCart->getSynchronyClientToken();

			try {
				$synchrony = new SynchronyApi($synchrony_api_keys['synchrony-merchant-number'], $synchrony_api_keys['synchrony-merchant-password'], $synchrony_api_keys['synchrony-test-acount']);
				$synchrony_status = $synchrony->getPaymentStatus($synchrony_client_token);
				if(!array_key_exists('AuthCode', $synchrony_status))
					throw new Exception('Error : Synchrony Card payment failed');
			}
			catch(Exception $e) {
				$ShoppingCart->clearSynchronyData();
				throw new Exception($e->getMessage());
			}
			
			$payment_authorization_details = array(
									'authCode' => $synchrony_status['AuthCode'], 	
									'transId' => $synchrony_status['transactionId'], 
									'refId' => $synchrony_status['ClientTransactionID'], 
									'full_response' => json_encode($synchrony_status),
									'amount' => $synchrony_status['TransactionAmount'], 
								);

			$checkout_fields = $ShoppingCart->getCheckoutFields();

			/* cart items & prices */
			$cart_prices = $ShoppingCart->getCartItemListWithPrices();
			if($cart_prices['total_price']['price'] != $payment_authorization_details['amount'])
				throw new Exception('Error : Total price of items different from the amount paid. Please contact us if your Airport Card has been charged');

			/* no tax from api */
			if($ShoppingCart->hasTaxApi() == 1 && ($cart_prices['tax_price']['price'] == 0 || $cart_prices['payment']['tax_price']['price'] == 'NA'))
				throw new Exception('Error : Order has no tax amount');

			/* add order to database */
			$order_id = $ShoppingCart->placeOrder($checkout_fields['billing'], $checkout_fields['shipping'], $checkout_fields['payment'], $cart_prices, $payment_authorization_details);
			$_SESSION['order_id'] = $order_id;

			echo json_encode(array('order_id' => $order_id));
			
			break;

		case 'getWellsFargoSessionToken':
			require_once('models/WellsFargoApi.php');

			$wellsfargo_api_keys = $ShoppingCart->getWellsFargoApiKeys();
			$wellsfargo = new WellsFargoApi($wellsfargo_api_keys['mode'], $wellsfargo_api_keys['gateway_entity_id'], $wellsfargo_api_keys['gateway_company_id'], $wellsfargo_api_keys['consumer_key'], $wellsfargo_api_keys['consumer_secret'], $wellsfargo_api_keys['certificate']);
			$wells_fargo_api_key = $wellsfargo->getApiKey();
			$session_data = $wellsfargo->getSessionToken($wells_fargo_api_key);
			$wells_fargo_session_token = $session_data['session_token'];
			$gateway_base_url = $session_data['gateway_base_url'];

			$session_token = json_encode(array('session_token' => $wells_fargo_session_token),  JSON_UNESCAPED_SLASHES);
			$sesion_response = '{"sessionToken":"' . str_replace('"', '\"', $session_token) . '", "gatewayBaseURL":"' . $gateway_base_url . '"}';

			header('Cache-Control: no-cache, no-store');
			header('Pragma: no-cache');
			header('Expires: 0');
			header('Strict-Transport-Security: max-age=31536000');
			
			echo $sesion_response;

			break;

		case 'placeOrderWithWellsFargo':
			require_once('models/WellsFargoApi.php');

			$ShoppingCart->lockCart();

			$checkout_fields = $ShoppingCart->getCheckoutFields();
			$ShoppingCart->validateBillingCheckoutFields($checkout_fields['billing']);
			$ShoppingCart->validateShippingCheckoutFields($checkout_fields['shipping']);
			$ShoppingCart->validatePaymentCheckoutFields($checkout_fields['payment']);

			/* cart items & prices */
			$cart_prices = $ShoppingCart->getCartItemListWithPrices();
			if(sizeof($cart_prices) == 0)
				throw new Exception('Error : No items in the shopping cart');

			/* no tax from api */
			if($ShoppingCart->hasTaxApi() == 1 && ($cart_prices['tax_price']['price'] == 0 || $cart_prices['payment']['tax_price']['price'] == 'NA'))
				throw new Exception('Error : Order has no tax amount');

			$last_4_account_no = $_POST['last_4_account_no'];
			$plan_no = $_POST['plan'];

			$wellsfargo_api_keys = $ShoppingCart->getWellsFargoApiKeys();
			$wellsfargo = new WellsFargoApi($wellsfargo_api_keys['mode'], $wellsfargo_api_keys['gateway_entity_id'], $wellsfargo_api_keys['gateway_company_id'], $wellsfargo_api_keys['consumer_key'], $wellsfargo_api_keys['consumer_secret'], $wellsfargo_api_keys['certificate']);
			$wells_fargo_api_key = $wellsfargo->getApiKey();
			$account_details = $wellsfargo->accountInquiry($wells_fargo_api_key, $wellsfargo_api_keys['merchant_number'], $checkout_fields['billing']['first_name'], $checkout_fields['billing']['last_name'], $last_4_account_no);
			$transaction_details = $wellsfargo->authorizePayment($wells_fargo_api_key, $wellsfargo_api_keys['merchant_number'], $cart_prices['total_price']['price'], $account_details['account_number'], $plan_no);

			$payment_authorization_details = array(
												'authCode' => $transaction_details['authorization_number'], 	
												'transId' => $transaction_details['client-request-id'], 
												'refId' => $transaction_details['client-request-id'], 
												'full_response' => json_encode($transaction_details),
												'amount' => $transaction_details['amount'], 
											);

			$ShoppingCart->updateWellsFargoBillingAddress($account_details);
			$checkout_fields = $ShoppingCart->getCheckoutFields();

			/* add order to database */
			$order_id = $ShoppingCart->placeOrder($checkout_fields['billing'], $checkout_fields['shipping'], $checkout_fields['payment'], $cart_prices, $payment_authorization_details);
			$_SESSION['order_id'] = $order_id;

			echo json_encode(array('order_id' => $order_id));

			break;

		case 'placeOrderWithAmazonPay':
			require_once('models/AmazonPayApi.php');

			$ShoppingCart->lockCart();

			$checkout_fields = $ShoppingCart->getCheckoutFields();
			$ShoppingCart->validateBillingCheckoutFields($checkout_fields['billing']);
			$ShoppingCart->validateShippingCheckoutFields($checkout_fields['shipping']);
			$ShoppingCart->validatePaymentCheckoutFields($checkout_fields['payment']);

			/* cart items & prices */
			$cart_prices = $ShoppingCart->getCartItemListWithPrices();
			if(sizeof($cart_prices) == 0)
				throw new Exception('Error : No items in the shopping cart');

			/* no tax from api */
			if($ShoppingCart->hasTaxApi() == 1 && ($cart_prices['tax_price']['price'] == 0 || $cart_prices['payment']['tax_price']['price'] == 'NA'))
				throw new Exception('Error : Order has no tax amount');

			$amazon_order_reference_id = $_POST['order_reference_id'];
			$cart_id = $ShoppingCart->getCartID();

			$amazonpay_api_keys = $ShoppingCart->getAmazonPayApiKeys();
			$amazon_pay = new AmazonPayApi($amazonpay_api_keys['mws-access-key-id'], $amazonpay_api_keys['mws-secret-key'], $amazonpay_api_keys['mws-seller-id'], $amazonpay_api_keys['amazonpay-test-acount']);
			$amazon_pay->setOrderReferenceDetails($amazon_order_reference_id, $cart_prices['total_price']['price'], $cart_id);
			$amazon_pay->confirmOrderReference($amazon_order_reference_id);
			$payment_authorization_details = $amazon_pay->authorize($amazon_order_reference_id, $cart_prices['total_price']['price'], $cart_id);

			/* add order to database */
			$order_id = $ShoppingCart->placeOrder($checkout_fields['billing'], $checkout_fields['shipping'], $checkout_fields['payment'], $cart_prices, $payment_authorization_details);
			$_SESSION['order_id'] = $order_id;

			echo json_encode(array('order_id' => $order_id));

			break;
	}
}
catch(Exception $e) {
	if($ShoppingCart->isCartLocked())
		$ShoppingCart->unlockCart();

	header('HTTP/1.1 400 Bad Request', true, 400);
    echo json_encode(array('error' => 1, 'message' => $e->getMessage()));
}


?>