<?php
session_start();

require_once('settings.php');
require_once(CONNDB_SCRIPT_PATH);
require_once('models/OrderDetails.php');
require_once('models/Email.php');

if(isset($_SESSION['order_id'])) {
    $order_id = $_SESSION['order_id'];
    unset($_SESSION['order_id']);
}
else {
    header('Location: ' . CART_PAGE);
}

try {
    $OrderDetails = new OrderDetails($hostname_connDB, $username_connDB, $password_connDB, $database_connDB);
    $order = $OrderDetails->getOrderDetails($order_id);

    if($order['paymenttype'] == 'CC')
        $payment_method = 'Credit Card';
    else if($order['paymenttype'] == 'PAYPAL')
        $payment_method = 'Paypal';
    else if($order['paymenttype'] == 'SYNCHRONY')
        $payment_method = 'Synchrony';
    else if($order['paymenttype'] == 'WELLSFARGO')
        $payment_method = 'Wells Fargo';
    else if($order['paymenttype'] == 'AMAZONPAY')
        $payment_method = 'Amazon Pay';
    else if($order['paymenttype'] == 'QUOTE')
        $payment_method = 'Quote';
    
    $Email = new Email();
    $email_html = $Email->getOrderConfirmationEmail($order);

    if($order['paymenttype'] == 'QUOTE')
        $recepient = $order['email_bcc'];
    else
        $recepient = ($order['localpickup'] == 1 ? $order['billing']['email'] : $order['shipemail']);
    $email_subject = 'Order Successfully Placed';
    $headers = array('MIME-Version: 1.0', 'Content-type: text/html;charset=UTF-8', 'From: ' . ($order['sitetitle'] . ' <' . $order['autoresponder'] . '>'), 'Content-Transfer-Encoding: base64');
    if($order['paymenttype'] != 'QUOTE')
        $headers[] = ('Bcc: ' . $order['email_bcc']);
    $email_sent = mail($recepient, $email_subject, chunk_split(base64_encode($email_html)), implode("\r\n", $headers));
    if(!$email_sent)
        throw new Exception('Error : Email could not be sent');
}
catch(Exception $e) {
    echo $e->getMessage();
    exit();
}

?>
<!DOCTYPE html>
<html>
<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
<meta name="viewport" content="width=device-width,initial-scale=1.0,maximum-scale=1.0,user-scalable=no">
<style type="text/css">

body {

}

#order-container {
    max-width: 1000px;
    margin: 80px auto;
    line-height: 1.5;
}

#success-header {
    font-size: 22px;
    font-weight: 700;
    margin: 0 0 15px 0;
}

#success-sub-header {
    font-size: 22px;
    margin: 0 0 0 35px;
}

#success-header img { 
    display: inline-block;
    vertical-align: middle;
    height: 25px;
    margin: 0 10px 0 0;
}

#order-id-container {
    margin: 40px 0 0 0;
}

#order-id {
    padding: 0 0 0 20px;
}

#order-items-container {
    margin: 60px 0 0 0;
}

.order-details-header {
    background-color: #F7F7F7;
    padding: 15px 20px;
    font-weight: 700;
    margin: 0 0 15px 0;
}

.order-details-header-title {
    display: inline-block;
    vertical-align: middle;
    padding: 0 0 5px 0;
    border-bottom: 3px solid #DA1206;
    text-transform: uppercase;
}

.order-row {
    margin: 0 0 15px 0;
}

.order-row-left {
    display: inline-block;
    vertical-align: top;
    width: 80%;
    padding: 0 0 0 20px;
    box-sizing: border-box;
}

.order-row-right {
    display: inline-block;
    vertical-align: top;
    width: 20%;
    text-align: right;
}

.order-item-name {
    white-space: nowrap;
    overflow: hidden;
    text-overflow: ellipsis;
    width: 100%;
}

.order-item-quantity {
    font-size: 14px;
    color: #676767;
    margin: 5px 0 0 0;
}

.discount-row {
    border-top: 2px solid #D8D6D6;
    padding: 15px 0 0 0;
}

.subtotal-row {
    border-top: 2px solid #D8D6D6;
    padding: 15px 0 0 0;
}

.total-row {
    font-weight: 700;
    border-top: 2px solid #D8D6D6;
    padding: 15px 0 0 0;
}

#shipping-address-container {
    margin: 60px 0 0 0;
}

#user-details-container {
    padding: 0 0 0 20px;
    box-sizing: border-box;
}

.user-details {
    display: inline-block;
    vertical-align: top;
    width: 26.66%;
    margin: 0 10% 0 0;
}

.user-details:last-child {
    margin: 0 !important;
}

.user-details-header {
    font-weight: 700;
    margin: 0 0 15px 0;
}

#requirements {
    margin: 40px 0 0 0;
    font-size: 16px;
    padding: 0 0 0 20px;
}

#requirements a {
    text-decoration: underline;
    color: #D22A27;
}

#print-receipt {
    border-radius: 8px;
    border: 2px solid #D22A27;
    padding: 15px 25px;
    font-size: 20px;
    font-family: inherit;
    outline: none;
    background-color: white;
    cursor: pointer;
    margin: 50px 0 0 20px;
    display: none;
}

@media print {
    
}

</style>
</head>

<body>

<div id="order-container">
    <h4 id="success-header"><img src="<?= IMG_DIRECTORY ?>/check-icon.svg" />Thank you for your <?= $order['paymenttype'] == 'QUOTE' ? 'quote' : 'order' ?>!</h4>
    <div id="success-sub-header">We’ve received your <?= $order['paymenttype'] == 'QUOTE' ? 'quote' : 'order' ?>. Our team will follow up with you within 24 hours.</div>
    <div id="order-id-container">
        <div class="order-details-header">
            <div class="order-details-header-title"><?= $order['paymenttype'] == 'QUOTE' ? 'Quote' : 'Order' ?> ID</div>
        </div>
        <div id="order-id"><?= $order_id ?></div>
    </div>
    <div id="order-items-container">
        <div class="order-details-header">
            <div class="order-details-header-title"><?= $order['paymenttype'] == 'QUOTE' ? 'Quote' : 'Order' ?> Details</div>
        </div>
        <div id="order-items">
            <?php

            $html = '';
            for($i=0; $i<sizeof($order['items']); $i++) {
                $html .= '<div class="order-row">';
                    $html .= '<div class="order-row-left">';
                        $html .= '<div class="order-item-name">' . $order['items'][$i]['details'] . '</div>';
                        $html .= '<div class="order-item-quantity">Qty: ' . $order['items'][$i]['qty'] . '</div>';
                    $html .= '</div>';
                    $html .= '<div class="order-row-right">' . money_format('$%i', $order['items'][$i]['qty']*$order['items'][$i]['price']) . '</div>';
                $html .= '</div>';
            }

            if($order['discount'] != NULL) {
                $html .= '<div class="order-row discount-row">';
                    $html .= '<div class="order-row-left">Discounts</div>';
                    $html .= '<div class="order-row-right">$' . $order['discount'] . '</div>';
                $html .= '</div>';
            }

            $html .= '<div class="order-row subtotal-row">';
                $html .= '<div class="order-row-left">Subtotal</div>';
                $html .= '<div class="order-row-right">$' . $order['subtotal'] . '</div>';
            $html .= '</div>';
            $html .= '<div class="order-row">';
                $html .= '<div class="order-row-left">Shipping</div>';
                $html .= '<div class="order-row-right">$' . $order['shipping'] . '</div>';
            $html .= '</div>';
            $html .= '<div class="order-row">';
                $html .= '<div class="order-row-left">Tax</div>';
                $html .= '<div class="order-row-right">$' . $order['tax'] . '</div>';
            $html .= '</div>';
            $html .= '<div class="order-row total-row">';
                $html .= '<div class="order-row-left">Total</div>';
                $html .= '<div class="order-row-right">$' . $order['grandtotal'] . '</div>';
            $html .= '</div>';

            echo $html;

            ?>
        </div>
    </div>
    <div id="shipping-address-container">
        <div class="order-details-header">
            <div class="order-details-header-title">Delivery</div>
        </div>
        <div id="user-details-container">
            <div class="user-details">
                <div class="user-details-header">Billing Address</div>
                <div><?= $order['billing']['fname'] . ' ' . $order['billing']['lname'] . '<br />' . $order['billing']['address1'] . ($order['billing']['address1'] != '' ? ', ' . $order['billing']['address2'] : '') . '<br />' . $order['billing']['city'] . '<br />' . $order['billing']['region'] . ' ' . $order['billing']['zip'] ?></div>
            </div><!--
         --><div class="user-details">
                <div class="user-details-header"><?= $order['localpickup'] == 1 ? 'Warehouse Pickup Address' : 'Delivery Address' ?></div>
                <div>
                    <?php

                    $html = '';
                    if($order['localpickup'] == 1)
                        $html = $order['pickup_store'];
                    else
                        $html = $order['shipfname'] . ' ' . $order['shiplname'] . '<br />' . $order['shipaddress1'] . ($order['shipaddress1'] != '' ? ', ' . $order['shipaddress2'] : '') . '<br />' . $order['shipcity'] . '<br />' . $order['shipregion'] . ' ' . $order['shipzip'];
                    
                    echo $html;
                    ?>
                </div>
            </div><!--
         --><div class="user-details">
                <div class="user-details-header">Payment Information</div>
                <div><?= $payment_method . '<br />' . 'Transaction ID : ' . $order['transactionid'] ?></div>
            </div>
        </div>
        <?php

        if($order['paymenttype'] != 'QUOTE') {
            $html = '';
            if($order['localpickup'] == 1) {
                $html = '<div id="requirements">We will call when your order is ready for collection, please bring the credit card used for this purchase and a valid form of identification.</div>';
            }

            echo $html;
        }

        ?>
    </div>
    <button id="print-receipt">Print Receipt</button>
</div>

<script>

document.querySelector("#print-receipt").addEventListener('click', function() {
    window.print();
});

</script>

</body>
</html>