<?php

class AmazonPayApi
{
	private $mws_access_key_id;

	private $mws_seller_id;

	private $mws_secret_key;

	private $is_sandbox;

	public function __construct($mws_access_key_id, $mws_secret_key, $mws_seller_id, $is_sandbox) {
		$this->mws_access_key_id = $mws_access_key_id;
		$this->mws_secret_key = $mws_secret_key;
		$this->mws_seller_id = $mws_seller_id;
		$this->is_sandbox = $is_sandbox;
	}

	public function setOrderReferenceDetails($order_reference_id, $grand_total, $cart_id) {
		$timestamp = gmdate('Y-m-d\TH:i:s\Z');

		$parameters = array();
		$parameters['AWSAccessKeyId'] = $this->mws_access_key_id;
		$parameters['Action'] = 'SetOrderReferenceDetails';
		$parameters['AmazonOrderReferenceId'] = $order_reference_id;
		$parameters['OrderReferenceAttributes.OrderTotal.Amount'] = $grand_total;
		$parameters['OrderReferenceAttributes.OrderTotal.CurrencyCode'] = 'USD';
		$parameters['OrderReferenceAttributes.SellerOrderAttributes.SellerOrderId'] = $cart_id;
		$parameters['SellerId'] = $this->mws_seller_id;
		$parameters['SignatureMethod'] = 'HmacSHA256';
		$parameters['SignatureVersion'] = '2';
		$parameters['Timestamp'] = rawurlencode($timestamp);
		$parameters['Version'] = '2013-01-01';

		$canonical_parameters = [];
		foreach($parameters as $key => $value) {
			$canonical_parameters[] = $key . "=" . $value;
		}
		$canonical_parameters = implode("&", $canonical_parameters);

		$canonical_request = array();
		$canonical_request[] = "POST";
		$canonical_request[] = "mws.amazonservices.com";
		$canonical_request[] = ($this->is_sandbox == 1 ? "/OffAmazonPayments_Sandbox/2013-01-01" : "/OffAmazonPayments/2013-01-01");
		$canonical_request[] = $canonical_parameters;
		$canonical_request = implode("\n", $canonical_request);

		$signature = urlencode(base64_encode(hash_hmac('sha256', $canonical_request, $this->mws_secret_key, true)));

		$query_params = $canonical_parameters . '&Signature=' . $signature;

		if($this->is_sandbox == 1)
			$api_url = 'https://mws.amazonservices.com/OffAmazonPayments_Sandbox/2013-01-01?' . $query_params;
		else
			$api_url = 'https://mws.amazonservices.com/OffAmazonPayments/2013-01-01';
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $api_url);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: x-www-form-urlencoded', 'Host: mws.amazonservices.com'));
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_POST, 1);
		$data = curl_exec($ch);
		$http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		if($http_code != 200) 
			throw new Exception('Error : Failed to set order reference details');
	}

	public function confirmOrderReference($order_reference_id) {
		$timestamp = gmdate('Y-m-d\TH:i:s\Z');

		$parameters = array();
		$parameters['AWSAccessKeyId'] = $this->mws_access_key_id;
		$parameters['Action'] = 'ConfirmOrderReference';
		$parameters['AmazonOrderReferenceId'] = $order_reference_id;
		$parameters['SellerId'] = $this->mws_seller_id;
		$parameters['SignatureMethod'] = 'HmacSHA256';
		$parameters['SignatureVersion'] = '2';
		$parameters['Timestamp'] = rawurlencode($timestamp);
		$parameters['Version'] = '2013-01-01';

		$canonical_parameters = [];
		foreach($parameters as $key => $value) {
			$canonical_parameters[] = $key . "=" . $value;
		}
		$canonical_parameters = implode("&", $canonical_parameters);

		$canonical_request = array();
		$canonical_request[] = "POST";
		$canonical_request[] = "mws.amazonservices.com";
		$canonical_request[] = ($this->is_sandbox == 1 ? "/OffAmazonPayments_Sandbox/2013-01-01" : "/OffAmazonPayments/2013-01-01");
		$canonical_request[] = $canonical_parameters;
		$canonical_request = implode("\n", $canonical_request);

		$signature = urlencode(base64_encode(hash_hmac('sha256', $canonical_request, $this->mws_secret_key, true)));

		$query_params = $canonical_parameters . '&Signature=' . $signature;

		if($this->is_sandbox == 1)
			$api_url = 'https://mws.amazonservices.com/OffAmazonPayments_Sandbox/2013-01-01?' . $query_params;
		else
			$api_url = 'https://mws.amazonservices.com/OffAmazonPayments/2013-01-01';
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $api_url);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: x-www-form-urlencoded', 'Host: mws.amazonservices.com'));
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_POST, 1);
		$data = curl_exec($ch);
		$http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		if($http_code != 200) 
			throw new Exception('Error : Failed to confirm order reference');
	}

	public function authorize($order_reference_id, $grand_total, $cart_id) {
		$timestamp = gmdate('Y-m-d\TH:i:s\Z');

		$parameters = array();
		$parameters['AWSAccessKeyId'] = $this->mws_access_key_id;
		$parameters['Action'] = 'Authorize';
		$parameters['AmazonOrderReferenceId'] = $order_reference_id;
		$parameters['AuthorizationAmount.Amount'] = $grand_total;
		$parameters['AuthorizationAmount.CurrencyCode'] = 'USD';	
		$parameters['AuthorizationReferenceId'] = $cart_id;
		$parameters['SellerId'] = $this->mws_seller_id;
		$parameters['SignatureMethod'] = 'HmacSHA256';
		$parameters['SignatureVersion'] = '2';
		$parameters['Timestamp'] = rawurlencode($timestamp);
		$parameters['Version'] = '2013-01-01';

		$canonical_parameters = [];
		foreach($parameters as $key => $value) {
			$canonical_parameters[] = $key . "=" . $value;
		}
		$canonical_parameters = implode("&", $canonical_parameters);

		$canonical_request = array();
		$canonical_request[] = "POST";
		$canonical_request[] = "mws.amazonservices.com";
		$canonical_request[] = ($this->is_sandbox == 1 ? "/OffAmazonPayments_Sandbox/2013-01-01" : "/OffAmazonPayments/2013-01-01");
		$canonical_request[] = $canonical_parameters;
		$canonical_request = implode("\n", $canonical_request);

		$signature = urlencode(base64_encode(hash_hmac('sha256', $canonical_request, $this->mws_secret_key, true)));

		$query_params = $canonical_parameters . '&Signature=' . $signature;

		if($this->is_sandbox == 1)
			$api_url = 'https://mws.amazonservices.com/OffAmazonPayments_Sandbox/2013-01-01?' . $query_params;
		else
			$api_url = 'https://mws.amazonservices.com/OffAmazonPayments/2013-01-01';
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $api_url);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: x-www-form-urlencoded', 'Host: mws.amazonservices.com'));
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_POST, 1);
		$data = curl_exec($ch);
		$http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		if($http_code != 200) 
			throw new Exception('Error : Failed to authorize payment');

		$doc = new DOMDocument();
		$doc->loadXML($data);

		$transaction_details = array(
									"authCode" => $doc->getElementsByTagName('AmazonAuthorizationId')->item(0)->nodeValue, 
									"transId" => $doc->getElementsByTagName('AuthorizationReferenceId')->item(0)->nodeValue, 
									"refId" => $doc->getElementsByTagName('AuthorizationReferenceId')->item(0)->nodeValue, 
									"full_response" => $data
								);

		return $transaction_details;
	}
}

?>