<?php

class AuthorizeApi
{
	private $login_id;

	private $transaction_key;

	private $is_sandbox;

	private $sandbox_url;

	private $production_url;

	public function __construct($login_id, $transaction_key, $is_sandbox = 0) {
		$this->login_id = $login_id;
		$this->transaction_key = $transaction_key;
		$this->is_sandbox = $is_sandbox;
		$this->sandbox_url = "https://apitest.authorize.net/xml/v1/request.api";
		$this->production_url = "https://api.authorize.net/xml/v1/request.api";
	}

	public function authorizePayment($billing, $shipping, $payment, $items, $cart_id, $cart_amount, $shipping_price, $tax_price) {
		$description = array();
		$line_items = array();
		for($i=0; $i<sizeof($items); $i++) {
			$description[] = $items[$i]["name"];
			$line_items[] = array("itemId" => $items[$i]["sku"], "name" => substr($items[$i]["sku"], 0, 30), "description" => substr(htmlspecialchars($items[$i]["name"]), 0, 254), "quantity" => $items[$i]["quantity"], "unitPrice" => $items[$i]["price"]);
		}
		$description = implode(" ", $description);

		$authorizePaymentArray = array();
		$authorizePaymentArray['createTransactionRequest'] = array();
		$authorizePaymentArray['createTransactionRequest']['merchantAuthentication'] = array("name" => $this->login_id, "transactionKey" => $this->transaction_key);
		$authorizePaymentArray['createTransactionRequest']['refId'] = $cart_id;
		$authorizePaymentArray['createTransactionRequest']['transactionRequest'] = array();
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['transactionType'] = "authCaptureTransaction";
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['amount'] = $cart_amount;
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['payment'] = array();
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['payment']['creditCard'] = array("cardNumber" => $payment["card_number"], "expirationDate" => $payment["card_year"] . "-" . $payment["card_month"], "cardCode" => $payment["card_ccv"]);
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['order'] = array("invoiceNumber" => $cart_id, "description" => substr($description, 0, 254));
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['lineItems'] = array();
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['lineItems']['lineItem'] = $line_items;
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['tax'] = array('amount' => $tax_price);
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['shipping'] = array('amount' => $shipping_price);
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['taxExempt'] = "false";
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['poNumber'] = $cart_id;
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['billTo'] = array();
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['billTo']['firstName'] = $billing['first_name'];
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['billTo']['lastName'] = $billing['last_name'];
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['billTo']['company'] = "";
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['billTo']['address'] = substr($billing['street_1'] . " " . $billing['street_2'], 0, 59);
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['billTo']['city'] = $billing['city'];
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['billTo']['state'] = $billing['state'];
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['billTo']['zip'] = $billing['zip'];
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['billTo']['country'] = $billing['country'];
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['billTo']['phoneNumber'] = $billing['phone'];
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['billTo']['faxNumber'] = "";
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['shipTo'] = array();
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['shipTo']['firstName'] = $shipping['first_name'];
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['shipTo']['lastName'] = $shipping['last_name'];
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['shipTo']['company'] = "";
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['shipTo']['address'] = substr($shipping['street_1'] . " " . $shipping['street_2'], 0, 59);
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['shipTo']['city'] = $shipping['city'];
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['shipTo']['state'] = $shipping['state'];
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['shipTo']['zip'] = $shipping['zip'];
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['shipTo']['country'] = $shipping['country'];
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['customerIP'] = $_SERVER['REMOTE_ADDR'];
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['transactionSettings'] = array();
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['transactionSettings']['setting'] = array();
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['transactionSettings']['setting'][] = array('settingName' => 'recurringBilling', 'settingValue' => 'false');
		$authorizePaymentArray['createTransactionRequest']['transactionRequest']['transactionSettings']['setting'][] = array('settingName' => 'duplicateWindow', 'settingValue' => '0');

		$api_url = $this->is_sandbox?$this->sandbox_url:$this->production_url;

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_POST,1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($authorizePaymentArray));
		curl_setopt($ch, CURLOPT_URL, $api_url);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array("Content-Type: application/json"));
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST,  2);
		curl_setopt($ch, CURLOPT_USERAGENT, $_SERVER['HTTP_USER_AGENT']);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		$response = curl_exec($ch);
		$http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		curl_close ($ch);

		$response = json_decode(preg_replace('/[\x00-\x1F\x80-\xFF]/', '', $response), true);

		if($http_code == 200) {
			if(array_key_exists("transactionResponse", $response)) {
				if(array_key_exists("errors", $response['transactionResponse'])) {
					throw new Exception($response['transactionResponse']['errors'][0]['errorText']);					
				}
				else if($response['messages']['resultCode'] == 'Error') {
					throw new Exception($response['messages']['message'][0]['text']);
				}
				else {
					$transaction_details = array("authCode" => $response['transactionResponse']['authCode'], "transId" => $response['transactionResponse']['transId'], "refId" => $response['transactionResponse']['refId'], "full_response" => json_encode($response));
					return $transaction_details;
				}
			}
			else {
				throw new Exception("Error : Transaction was not approved. Check payment details provided");			
			}
		}
		else {
			throw new Exception("Error : Error Processing Request");
		}
	}
}

?>