<?php

class PaypalExpressCheckoutApi
{
	private $is_sandbox;

	private $client_id;

	private $client_secret;

	public function __construct($client_id, $client_secret, $is_sandbox = 0) {
		$this->client_id = $client_id;
		$this->client_secret = $client_secret;
		$this->is_sandbox = $is_sandbox;
	}

	public function createOrder($shipping, $items, $cart_id, $cart_amount) {
		$order_details = array(
							'reference_id' => $cart_id,
							'amount' => array(
											'currency_code' => 'USD', 
											'value' => $cart_amount['total'], 
											'breakdown' => array(
																'item_total' => array('currency_code' => 'USD', 'value' => $cart_amount['subtotal']),
																'tax_total' => array('currency_code' => 'USD', 'value' => $cart_amount['tax']),
																'shipping' => array('currency_code' => 'USD', 'value' => $cart_amount['shipping']),
															),
										),
							'shipping' => array(
											'name' => array('full_name' => $shipping['name']),
											'address' => array(
															'address_line_1' => $shipping['street_1'],
															'address_line_2' => $shipping['street_2'],
															'admin_area_2' => $shipping['city'],
															'admin_area_1' => $shipping['state'],
															'postal_code' => $shipping['zip'],
															'country_code' => $shipping['country']
														)
										),
							'items' => array()
						);
		for($i=0; $i<sizeof($items); $i++) {
			$line_item = array('name' => substr($items[$i]['name'], 0, 127), 'quantity' => $items[$i]['quantity'], 'unit_amount' =>  array('currency_code' => 'USD', 'value' => $items[$i]['price']), 'sku' => $items[$i]['sku']);
			$order_details['items'][] = $line_item;
		}

		if($this->is_sandbox == 1)
			$api_url = 'https://api.sandbox.paypal.com/v2/checkout/orders';
		else
			$api_url = 'https://api.paypal.com/v2/checkout/orders';
		
		$curl_post = array('intent' => 'CAPTURE', 'purchase_units' => array($order_details));
		$ch = curl_init();		
		curl_setopt($ch, CURLOPT_URL, $api_url);		
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));
		curl_setopt($ch, CURLOPT_USERPWD, $this->client_id . ":" . $this->client_secret);
		curl_setopt($ch, CURLOPT_POST, 1);		
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($curl_post));
		$data = json_decode(curl_exec($ch), true);
		$http_code = curl_getinfo($ch,CURLINFO_HTTP_CODE);		
		if($http_code != 201) 
			throw new Exception('Error : Failed to create Paypal order');
			
		return $data['id'];
	}

	public function createOrderCapture($paypal_order_id) {
		if($this->is_sandbox == 1)
			$api_url = 'https://api.sandbox.paypal.com/v2/checkout/orders/'. $paypal_order_id . '/capture';
		else
			$api_url = 'https://api.paypal.com/v2/checkout/orders/'. $paypal_order_id . '/capture';
		
		$ch = curl_init();		
		curl_setopt($ch, CURLOPT_URL, $api_url);		
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));
		curl_setopt($ch, CURLOPT_USERPWD, $this->client_id . ":" . $this->client_secret);
		curl_setopt($ch, CURLOPT_POST, 1);		
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		$data = json_decode(curl_exec($ch), true);
		$http_code = curl_getinfo($ch,CURLINFO_HTTP_CODE);		
		if($http_code != 201) 
			throw new Exception('Error : Failed to capture Paypal order');

		$authorization_details = array(
									'authCode' => $data['purchase_units'][0]['payments']['captures'][0]['id'], 
									'transId' => $data['id'], 
									'refId' => $data['purchase_units'][0]['reference_id'], 
									'full_response' => json_encode($data),
									'amount' => $data['purchase_units'][0]['payments']['captures'][0]['amount']['value'],
								);
			
		return $authorization_details;
	}
}

?>