<?php

class PaypalWebsitePaymentsProApi
{
	private $is_sandbox;

	private $nvp_username;

	private $nvp_password;

	private $nvp_signature;

	public function __construct($is_sandbox, $nvp_username, $nvp_password, $nvp_signature) {
		$this->is_sandbox = $is_sandbox;
		$this->nvp_username = $nvp_username;
		$this->nvp_password = $nvp_password;
		$this->nvp_signature = $nvp_signature;
	}

	public function capturePayment($billing, $shipping, $payment, $items, $cart_id, $cart_amount) {
		if($this->is_sandbox == 1)
			$api_url = 'https://api-3t.sandbox.paypal.com/nvp';
		else
			$api_url = 'https://api-3t.paypal.com/nvp';

		$params = array(
						'USER' => $this->nvp_username,
						'PWD' => $this->nvp_password,
						'SIGNATURE' => $this->nvp_signature,					
						'METHOD' => 'DoDirectPayment',
						'VERSION' => '204.0',
						'PAYMENTACTION' => 'Sale',
						'IPADDRESS' => $_SERVER['REMOTE_HOST'],
						'RETURNFMFDETAILS' => '1',
						'ACCT' => $payment['card_number'],
						'EXPDATE' => ($payment['card_month'] . $payment['card_year']),
						'CVV2' => $payment['card_ccv'],
						'FIRSTNAME' => substr($billing['first_name'], 0, 29),
						'LASTNAME' => substr($billing['last_name'], 0, 29),
						'STREET' => substr($billing['street_1'], 0, 29),
						'STREET2' => substr($billing['street_2'], 0, 29),
						'CITY' => substr($billing['city'], 0, 19),
						'STATE' => $billing['state'],
						'ZIP' => $billing['zip'],
						'COUNTRYCODE' => $billing['country'],
						'AMT' => $cart_amount,
						'CURRENCYCODE' => 'USD',
					);	
		if($shipping['delivery_type'] != 'PICKUP') {
			$params['SHIPTONAME'] = substr($shipping['name'], 0, 29);
			$params['SHIPTOSTREET'] = substr($shipping['street_1'], 0, 29);
			$params['SHIPTOSTREET2'] = substr($shipping['street_2'], 0, 29);
			$params['SHIPTOCITY'] = substr($shipping['city'], 0, 19);
			$params['SHIPTOSTATE'] = $shipping['state'];
			$params['SHIPTOZIP'] = $shipping['zip'];
			$params['SHIPTOCOUNTRY'] = $shipping['country'];
		}

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_POST,1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));
		curl_setopt($ch, CURLOPT_URL, $api_url);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		$response = curl_exec($ch);
		parse_str($response, $response);
		$http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		curl_close ($ch);
		if($http_code != 200)
			throw new Exception('Error: Payment could not be processed');
		
		if($response['ACK'] == 'Success')
			$transaction_details = array("authCode" => $response['TRANSACTIONID'], "transId" => $response['TRANSACTIONID'], "refId" => $response['CORRELATIONID'], "full_response" => json_encode($response));
		else
			throw new Exception('Error: Payment Failed - ' . $response['L_ERRORCODE0'] . ' - ' . $response['L_SHORTMESSAGE0']);
			
		return $transaction_details;
	}
}

?>