<?php

class ProductDetails
{
	private $conndb;

	private $conndb_products;

	private $sku;

	private $id;

	private $pkCategory;

	private $pkBrand;

	private $Sale;

	private $MSRP;

	private $warranties;

	private $mh_slug;

	private $cg_slug;

	private $c_slug;

	private $addons;

	private $is_outlet;

	private $options;

	public function __construct($conndb, $conndb_products, $mh_slug = NULL, $cg_slug = NULL, $c_slug = NULL, $sku = NULL, $product_id = NULL, $is_outlet = 0) {
		$this->conndb = $conndb;
		$this->conndb_products = $conndb_products;
		
		if($product_id != NULL) {
			$query = "SELECT 
					tbl_products.SKU, 
					tbl_products.mhSlug, 
					tbl_products.cgSlug, 
					tbl_products.cSlug 
				FROM tbl_products WHERE id=" . $this->conndb->real_escape_string($product_id);

			$result = $this->conndb->query($query);
			if(!$result)
				throw new Exception($this->conndb->error);
			if($result->num_rows == 0)
				throw new Exception('Error : No product with this ID');

			$row = $result->fetch_assoc();
			$mh_slug = $row['mhSlug'];
			$cg_slug = $row['cgSlug'];
			$c_slug = $row['cSlug'];
			$sku = $row['SKU'];

			if($is_outlet == 1) {
				$query = "SELECT outlet FROM ModelPrice WHERE SKU='" . $this->conndb->real_escape_string($sku) . "'";
				$result = $this->conndb->query($query);
				if(!$result)
					throw new Exception($this->conndb->error);
				if($result->num_rows == 0)
					throw new Exception('Error : No product with this SKU');
				$row = $result->fetch_assoc();
				$outlet = $row['outlet'];

				if($outlet != 1)
					throw new Exception('Error : No outlet product with this ID');
			}
		}

		$this->sku = $sku;
		$this->mh_slug = $mh_slug;
		$this->cg_slug = $cg_slug;
		$this->c_slug = $c_slug;
		$this->is_outlet = $is_outlet;

		$this->pkCategory = NULL;
		$this->Sale = NULL;
		$this->MSRP = NULL;
		$this->warranties = NULL;
		$this->addons = NULL;
		$this->options = NULL;
		$this->id = NULL;
	}

	public function getProductBasicDetails() {
		$query = "SELECT 
					tbl_products.id, 
					tbl_products.SKU, 
					tbl_products.mhDescription, 
					tbl_products.mhSlug, 
					tbl_products.cgDescription, 
					tbl_products.cgSlug, 
					tbl_products.CatDescription, 
					tbl_products.cSlug, 
					tbl_products.pkBrand, 
					tbl_products.pkCategory, 
					IF(tbl_products.CustomDescription <> '' AND tbl_products.CustomDescription IS NOT NULL ,tbl_products.CustomDescription, tbl_products.ModelDescription) AS ModelDescription,
					if(tbl_products.thumb LIKE 'http%',tbl_products.thumb, concat('/uploads/products/',tbl_products.thumb)) AS thumb,
					if(tbl_products.large LIKE 'http%',tbl_products.large, concat('/uploads/products/',tbl_products.large)) AS large, 
					Brand_Data.CommonBrandName, 
					Brand_Data.CommonBrandID, 
					ModelPrice.OutletPrice, 
					ModelPrice.MSRP, 
					IF((ModelPrice.Sale = 0), ModelPrice.MSRP, IF((ModelPrice.MSRP = 0), ModelPrice.Sale, IF((ModelPrice.Sale > ModelPrice.MSRP), ModelPrice.MSRP, ModelPrice.Sale))) AS Sale 
				FROM tbl_store_brands 
					INNER JOIN Brand_Data ON (tbl_store_brands.brandID = Brand_Data.pkID) 
					INNER JOIN tbl_products ON (tbl_store_brands.categoryID = tbl_products.pkCategory AND tbl_store_brands.brandID = tbl_products.pkBrand AND tbl_products.mhSlug='" . $this->conndb->real_escape_string($this->mh_slug) . "' AND tbl_products.cgSlug='" . $this->conndb->real_escape_string($this->cg_slug) . "' AND tbl_products.cSlug='" . $this->conndb->real_escape_string($this->c_slug) . "' AND REPLACE(tbl_products.SKU,'/','-')='" . $this->conndb->real_escape_string($this->sku) . "') 
					INNER JOIN ModelPrice ON (ModelPrice.SKU = tbl_products.SKU) 
				GROUP BY tbl_products.SKU";

		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);

		if($result->num_rows == 0)
			throw new Exception('Error : Product does not exist');

		$row = $result->fetch_assoc();

		if($this->is_outlet == 1) {
			$row['Sale'] = $row['OutletPrice'];
			$row['ModelDescription'] = $row['ModelDescription'] . ' [OUTLET]';
		}

		if($row['Sale'] == '0.00' || $row['Sale'] == NULL)
			$row['prices_available'] = 0;
		else
			$row['prices_available'] = 1;

		if($row['MSRP'] > $row['Sale'])
			$row['discount'] = $row['MSRP'] - $row['Sale'];
		else
			$row['discount'] = 0.00;

		$row['url'] = strtolower(substr($_SERVER['SERVER_PROTOCOL'], 0, strpos($_SERVER['SERVER_PROTOCOL'], '/'))) . '://' . $_SERVER['SERVER_NAME'] . '/' . $row['mhSlug'] . '/' . $row['cgSlug'] . '/' . $row['cSlug'] . '/' . $row['SKU'];

		$this->pkCategory = $row['pkCategory'];
		$this->pkBrand = $row['pkBrand'];
		$this->MSRP = $row['MSRP'];
		$this->Sale = $row['Sale'];
		$this->id = $row['id'];

		return $row;
	}

	public function getProductWarrantyDetails() {
		if($this->warranties != NULL)
			return $this->warranties;

		if($this->pkCategory == NULL) {
			$this->getProductBasicDetails();
		}

		if($this->Sale == -1) {
			$this->warranties = array();
			return $this->warranties;
		}

		$query = "SELECT  
					warranties.war_code, 
					SUM(warranties.2_year) as 2_year, 
					SUM(warranties.3_year) as 3_year, 
					SUM(warranties.4_year) as 4_year, 
					SUM(warranties.5_year) as 5_year, 
					warranties.description, 
					warranties.id, 
					warranties.service_type 
				FROM warranties 
					INNER JOIN categories_warranty AS categories_warranty ON categories_warranty.warranties_id = warranties.id 
				WHERE " . $this->Sale . " BETWEEN warranties.min_price and warranties.max_price AND categories_warranty.pkID=" . $this->pkCategory . " GROUP BY warranties.id";

		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);
		if($result->num_rows == 0)
			return array();

		$row = $result->fetch_assoc();
		$warranties = array();
		if($row['2_year'] != '')
			$warranties['2_year'] = array('name' => '2 Years', 'description' => $row['description'], 'price' => $row['2_year'], 'war_code' => $row['war_code'], 'id' => $row['id'], 'tooltip' => 'Keep your furniture looking great by protecting it against stains and damages with a 2 year Montage Protection Plan. Kids, pets, entertaining, and daily life can take a toll on furniture. A Montage Protection Plan covers against accidental rips, tears, burns, stains, breakage of frames and mechanisms, pet damage, seam separation, or sun fading once it occurs. Add Montage to your purchase today and have peace of mind knowing your furniture will look great and work well for years to come. <a href="/about-us" target="_blank">Learn More</a>');
		if($row['3_year'] != '')
			$warranties['3_year'] = array('name' => '3 Years', 'description' => $row['description'], 'price' => $row['3_year'], 'war_code' => $row['war_code'], 'id' => $row['id'], 'tooltip' => 'Keep your furniture looking great by protecting it against stains and damages with a 3 year Montage Protection Plan. Kids, pets, entertaining, and daily life can take a toll on furniture. A Montage Protection Plan covers against accidental rips, tears, burns, stains, breakage of frames and mechanisms, pet damage, seam separation, or sun fading once it occurs. Add Montage to your purchase today and have peace of mind knowing your furniture will look great and work well for years to come. <a href="/about-us" target="_blank">Learn More</a>');
		if($row['4_year'] != '')
			$warranties['4_year'] = array('name' => '4 Years', 'description' => $row['description'], 'price' => $row['4_year'], 'war_code' => $row['war_code'], 'id' => $row['id'], 'tooltip' => 'Keep your furniture looking great by protecting it against stains and damages with a 4 year Montage Protection Plan. Kids, pets, entertaining, and daily life can take a toll on furniture. A Montage Protection Plan covers against accidental rips, tears, burns, stains, breakage of frames and mechanisms, pet damage, seam separation, or sun fading once it occurs. Add Montage to your purchase today and have peace of mind knowing your furniture will look great and work well for years to come. <a href="/about-us" target="_blank">Learn More</a>');
		if($row['5_year'] != '')
			$warranties['5_year'] = array('name' => '5 Years', 'description' => $row['description'], 'price' => $row['5_year'], 'war_code' => $row['war_code'], 'id' => $row['id'], 'tooltip' => 'Keep your furniture looking great by protecting it against stains and damages with a 5 year Montage Protection Plan. Kids, pets, entertaining, and daily life can take a toll on furniture. A Montage Protection Plan covers against accidental rips, tears, burns, stains, breakage of frames and mechanisms, pet damage, seam separation, or sun fading once it occurs. Add Montage to your purchase today and have peace of mind knowing your furniture will look great and work well for years to come. <a href="/about-us" target="_blank">Learn More</a>');

		$this->warranties = $warranties;
	
		return $warranties;
	}

	public function getProductAddons() {
		if($this->addons != NULL)
			return $this->addons;

		if($this->pkCategory == NULL) {
			$this->getProductBasicDetails();
		}

		$query = "SELECT tbl_addons.addonDescription AS description,tbl_addons.pkID AS id,tbl_addons.sPrice AS price,tbl_addons.taxable,tbl_addons.SKU AS sku FROM tbl_addons INNER JOIN tbl_addons_cat ON tbl_addons.pkID=tbl_addons_cat.addon_id WHERE tbl_addons_cat.category='" . $this->c_slug . "' AND tbl_addons.Active=1";
		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);
		if($result->num_rows == 0)
			return array();

		$addons = array();
		for($i=0; $i<$result->num_rows; $i++) {
			$row = $result->fetch_assoc();
			$addons[] = $row;
		}

		$this->addons = $addons;

		return $addons;
	}

	public function getProductOptions() {
		if($this->options != NULL)
			return $this->options;

		if($this->pkCategory == NULL) {
			$this->getProductBasicDetails();
		}

		$query = "SELECT pot.id AS option_type_id,pot.options AS option_type,GROUP_CONCAT(pol.id separator '@@') AS optionList_id,GROUP_CONCAT(pol.option_description separator '@@') AS option_description, GROUP_CONCAT(pol.cost separator '@@') AS cost, GROUP_CONCAT(pol.SKU separator '@@') AS optionList_SKU FROM tbl_product_options_list pol INNER JOIN tbl_products p ON p.SKU = pol.SKU INNER JOIN  tbl_product_options pot ON (pot.id = pol.tbl_option_id AND pot.Active = 1) WHERE p.id=" . $this->id . " GROUP BY pol.tbl_option_id";
		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);
		if($result->num_rows == 0)
			return array();

		$options = array();
		for($i=0; $i<$result->num_rows; $i++) {
			$row = $result->fetch_assoc();
			
			$option_type = $row['option_type'];
			$option_type_id = $row['option_type_id'];
			$option_list_description = explode("@@", $row['option_description']);
			$option_list_cost = explode("@@", $row['cost']);
			$option_list_id = explode("@@", $row['optionList_id']);
			$option_list_sku = explode("@@", $row['optionList_SKU']);

			$option = array('option_type' => $option_type, 'option_type_id' => $option_type_id, 'options_list' => array());
			for($j=0; $j<sizeof($option_list_description); $j++) {
				$option['options_list'][] = array('title' => $option_list_description[$j], 'price' => $option_list_cost[$j], 'id' => $option_list_id[$j], 'sku' => $option_list_sku[$j]);
			}

			$options[] = $option;
		}

		$this->options = $options;

		return $options;
	}
}

?>