<?php

class SiteShoppingCart extends ShoppingCart
{
	public function __construct($hostname_connDB, $username_connDB, $password_connDB, $database_connDB, $has_shipping_with_install = 0, $delivery_titles, $has_tax_api, $cc_gateway) {
		parent::__construct($hostname_connDB, $username_connDB, $password_connDB, $database_connDB, $has_shipping_with_install, $delivery_titles, $has_tax_api, $cc_gateway);
	}

	/* check whether zip code is allowed, and set zip code */
	public function checkZipCode($zip_code, $delivery_type = NULL) {
		/* no zip code check for quote mode */
		if($this->isCartMode() == 1) {
			$query = "SELECT ZIP FROM ZipCodes WHERE ZIP='" . $this->conndb->real_escape_string($zip_code) . "'";
			$result = $this->conndb->query($query);
			if(!$result)
				throw new Exception($this->conndb->error);

			if($result->num_rows == 0)
				throw new Exception('Unfortunately, you are outside of our standard delivery area. If you would like us to provide a shipping quote for this item, please contact us.');
		}

		$this->setZipCode($zip_code, $delivery_type);
	}

	/* return the cart shipping price */
	protected function getCartShippingPrice($products_categories, $cart_subtotal_price) {
		/* no shipping price for quote mode */
		if($this->isCartMode() == 0) {
			return 0.00;
		}

		if($this->getDeliveryType() == 'PICKUP') {
			return 0.00;
		}
		else {
			$zip_code = $this->getZipCode();

			$query = "SELECT delivery_charge FROM ZipCodes WHERE Zip=" . $zip_code;
			$result = $this->conndb->query($query);
			if(!$result)
				throw new Exception($this->conndb->error);
			if($result->num_rows == 0)
				throw new Exception('Error: No delivery charges found');

			$row = $result->fetch_assoc();
			$shipping_amount_total = $row['delivery_charge'];
		}

		return $shipping_amount_total;
	}

	/* check whether shipping zip code & state match */
	public function checkShippingZipStateMatch($zip_code, $state) {
		$query = "SELECT ZIP,State FROM ZipCodes WHERE ZIP='" . $this->conndb->real_escape_string($zip_code) . "'";
		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);
		$row = $result->fetch_assoc();
		if($row['State'] != $state)
			throw new Exception('Error : Shipping zip/state do not match');
	}

	/* return the cart tax price bases on tax rates from database */
	protected function getCartTaxPrice($cart_taxable_price) {
		if($this->isCartMode() == 0) {
			return 0.00;
		}

		if($this->checkout_fields['shipping']['delivery_type'] == 'PICKUP') {
			$store_id = $this->getPickupStore();
			$store_details = $this->getPickupStoreDetails($store_id);
			$tax_zip = $store_details['zip'];
		}
		else {
			$tax_zip = $this->getZipCode();
		}

		$query = "SELECT tax_rate FROM ZipCodes WHERE Zip=" . $tax_zip;
		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);
		if($result->num_rows == 0)
			throw new Exception('Error: No tax rate found');

		$row = $result->fetch_assoc();
		$tax_rate = $row['tax_rate']/100;

		$total_tax = $tax_rate*$cart_taxable_price;

		return $total_tax;
	}

	/* return cart tax based on Taxjar API */
	protected function getCartTaxPriceViaApi($grand_total, $shipping, $fresh_request = 0) {
		if($this->getDeliveryType() == 'PICKUP') {
			$store_id = $this->getPickupStore();
			$store_details = $this->getPickupStoreDetails($store_id);
			
			$from_address = array('from_zip' => $store_details['zip'], 'from_state' => $store_details['state'], 'from_city' => $store_details['city'], 'from_street' => $store_details['address']);
			$to_address = array('to_zip' => $store_details['zip'], 'to_state' => $store_details['state'], 'to_city' => $store_details['city'], 'to_street' => $store_details['address']);
		}
		else {
			$checkout_fields = $this->getCheckoutFields();
			if($checkout_fields['shipping']['zip'] != NULL && $checkout_fields['shipping']['state'] != NULL && $checkout_fields['shipping']['city'] != NULL && $checkout_fields['shipping']['street_1'] != NULL) {
				$to_address = array('to_zip' => $checkout_fields['shipping']['zip'], 'to_state' => $checkout_fields['shipping']['state'], 'to_city' => $checkout_fields['shipping']['city'], 'to_street' => $checkout_fields['shipping']['street_1']);

				$closest_store = $this->getClosestPickupStore($to_address['to_state'], $to_address['to_zip']);
				$from_address = array('from_zip' => $closest_store['zip'], 'from_state' => $closest_store['state'], 'from_city' => $closest_store['city'], 'from_street' => $closest_store['address']);
			}
			else {
				return 'NA';
			}
		}

		/* check whether api parameters are the same before sending api request again */ 
		if($fresh_request == 0) {
			$taxjar_last_request = $this->getTaxApiLastRequest();
			$taxjar_last_zip = $taxjar_last_request['last_zip'];
			$taxjar_last_state = $taxjar_last_request['last_state'];
			$taxjar_last_city = $taxjar_last_request['last_city'];
			$taxjar_last_street = $taxjar_last_request['last_street'];
			$taxjar_last_grandtotal = $taxjar_last_request['last_grandtotal'];
			$taxjar_last_shipping = $taxjar_last_request['last_shipping'];
			$taxjar_last_sales_tax = $taxjar_last_request['last_sales_tax'];

			if($taxjar_last_zip == $to_address['to_zip'] && $taxjar_last_state == $to_address['to_state'] && $taxjar_last_city == $to_address['to_city'] && $taxjar_last_street == $to_address['to_street'] && $taxjar_last_grandtotal == $grand_total && $taxjar_last_shipping == $shipping)
				return $taxjar_last_sales_tax;
		}

		$taxjar = new TaxjarApi();
		$sales_tax_response = $taxjar->calculateSalesTax($to_address, $from_address, $shipping, $grand_total);
		$sales_tax = $sales_tax_response['tax'];
		$sales_tax_full_response = $sales_tax_response['full_response'];
		
		$this->setTaxApiLastRequest($to_address, $grand_total, $shipping, $sales_tax, $sales_tax_full_response);

		return $sales_tax;
	}

	/* get terms & conditions CMS page content */
	public function getTermsAndConditions() {
		$query = "SELECT details AS html,sidebar AS css FROM tbl_pages WHERE slug='sale-terms'";
		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);

		if($result->num_rows == 0) {
			return array('html' => '', 'css' => '');
		}

		$row = $result->fetch_assoc();
		return $row;
	}

	/* get all pickup stores allowed */
	public function getAllPickupStores() {
		$query = "SELECT id, CONCAT(name, ' ', address, ', ', city, ' ', state, ' ', zip) AS collectionpoint, address, city, state, zip FROM tbl_stores ORDER BY state,city,name";
		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);

		$pickup_stores = array();
		for($i=0; $i<$result->num_rows; $i++) {
			$row = $result->fetch_assoc();
			$pickup_stores[] = $row;
		}

		return $pickup_stores;
	}

	/* get details of a specific pickup store */
	public function getPickupStoreDetails($store_id) {
		$query = "SELECT id,name,address,city,state,zip FROM tbl_stores WHERE id=" . $this->conndb->real_escape_string($store_id);
		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);

		if($result->num_rows == 0)
			throw new Exception('Error: Pickup store is not valid');

		$row = $result->fetch_assoc();

		return $row;
	}

	/* check whether given store is a valid pickup store */ 
	public function checkPickupStore($store_id) {
		$query = "SELECT id FROM tbl_stores WHERE id=" . $this->conndb->real_escape_string($store_id);
		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);

		if($result->num_rows == 0)
			throw new Exception('Error: Pickup store is not valid');

		$this->setPickupStore($store_id);
	}

	/* get closest pickup store based on given zip */
	public function getClosestPickupStore($state, $zip) {
		$all_stores = $this->getAllPickupStores();

		$stores_in_state = array();
		for($i=0; $i<sizeof($all_stores); $i++) {
			if($all_stores[$i]['state'] == $state) {
				$stores_in_state[] = $all_stores[$i];
			}
		}

		$zip_difference = array();
		for($i=0; $i<sizeof($stores_in_state); $i++) {
			$zip_difference[] = abs((int)$stores_in_state[$i]['zip'] - (int)$zip);
		}

		$min = min($zip_difference);
		$index = array_search($min, $zip_difference);

		return $stores_in_state[$index];
	}

	/* get the logged-in client id */
	public function getClientID() {
		if(isset($_SESSION['clientID']))
			return $_SESSION['clientID'];
		else
			return -1;
	}

	/* get client billing address */
	public function getClientBillingAddress($client_id) {
		$query = "SELECT email,fname AS first_name,lname AS last_name,address1 AS street_1,address2 AS street_2,city,region AS state,zip,phone FROM tbl_clients WHERE id=" . $client_id;
		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);
		if($result->num_rows == 0)
			throw new Exception('Error : Client does not exist');
		$billing = $result->fetch_assoc();

		return $billing;
	}

	/* get the default zip code if user has not entered a zip code while adding product to cart */
	public function getDefaultZipcode() {
		$query = "SELECT ZIP FROM ZipCodes LIMIT 0,1";
		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);
		if($result->num_rows == 0)
			$default_zipcode = '91801';
		else {
			$row = $result->fetch_assoc();
			$default_zipcode = $row['ZIP'];
		}

		return $default_zipcode;
	}
}

class ShoppingCart
{
	/* database handle */
	protected $conndb;

	/* delivery fields for the current cart */
	protected $delivery_fields;

	/* delivery titles */
	protected $delivery_titles;

	/* cart items */
	protected $session_items;

	/* cart id */
	protected $cart_id;

	/* checkout fields for the current cart */
	protected $checkout_fields;

	/* flag whether to take data from session or make a fresh database query */
	protected $use_cached_info;

	/* paypal ordered created during paypal express checkout api */
	protected $paypal_order_id;

	/* token created during synchrony api */
	protected $synchrony_client_token;

	/* api response during synchrony api */
	protected $synchrony_verification_response;

	/* coupon id currently being used in cart */
	protected $coupon_id;

	/* whether taxjar api is enabled or not */ 
	protected $has_tax_api;

	/* whether cart mode or quote mode is activated */ 
	protected $is_cart_mode;

	/* cart locked for any changes to items added */
	protected $cart_locked;

	/* current credit card gateway api to be used */
	protected $cc_gateway;

	public function __construct($mysql_hostname, $mysql_username, $mysql_password, $mysql_database, $has_delivery_with_install, $delivery_titles, $has_tax_api, $cc_gateway) {
		$conndb = new mysqli($mysql_hostname, $mysql_username, $mysql_password, $mysql_database);
		if($conndb->connect_errno)
			throw new Exception('Error : Server failed. It might be possible that your DB credentials are wrong');
		$conndb->set_charset("utf8");
		$this->conndb = $conndb;

		$this->delivery_titles = array('SHIP' => $delivery_titles['SHIP'], 'SHIP-INSTALL' => $delivery_titles['SHIP-INSTALL'], 'PICKUP' => $delivery_titles['PICKUP']);
		$this->use_cached_info = 0;
		$is_cart_mode = $this->isCartModeEnabled();

		if(!isset($_SESSION['cart'])) {
			if(!isset($_COOKIE['shipping_zip'])) {
				$default_zipcode = $this->getDefaultZipcode();
			}

			$_SESSION['cart'] = array(
									'items' => '[]', 
									'cart_id' => time() . rand(11111, 99999),
									'subtotal' => 0.00,
									'checkout_fields' => array(
														'billing' => array(
																		'name' => NULL,
																		'first_name' => NULL,
																		'last_name' => NULL,
																		'email' => NULL,
																		'phone' => NULL,
																		'street_1' => NULL,
																		'street_2' => NULL,
																		'city' => NULL,
																		'state' => NULL,
																		'zip' => NULL,
																		'country' => NULL
																	),
														'shipping' => array(
																		'delivery_type' => 'SHIP',
																		'pickup_store' => NULL,
																		'name' => NULL,
																		'first_name' => NULL,
																		'last_name' => NULL,
																		'email' => NULL,
																		'phone' => NULL,
																		'street_1' => NULL,
																		'street_2' => NULL,
																		'city' => NULL,
																		'state' => NULL,
																		'zip' => NULL,
																		'country' => NULL
																	),
														'payment' => array(
																		'type' => ($is_cart_mode == 0 ? 'QUOTE' : NULL),
																		'card_number' => NULL,  
																		'card_name' => NULL, 
																		'card_ccv' => NULL, 
																		'card_month' => NULL, 
																		'card_year' => NULL,
																		'cc_gateway' => NULL
																	)
													),
									'delivery' => array(
													'delivery_type' => 'SHIP',
													'pickup_store' => NULL,
													'delivery_zip' => (isset($_COOKIE['shipping_zip']) ? $_COOKIE['shipping_zip'] : $default_zipcode),
													'has_delivery_with_install' => $has_delivery_with_install,
												),
									'paypal_order_id' => NULL,
									'synchrony_client_token' => NULL,
									'synchrony_verification_response' => NULL,
									'coupon_id' => NULL,
									'tax_api' => array(
													'enabled' => $has_tax_api,
													'params' => array(
														'last_zip' => NULL,
														'last_state' => NULL,
														'last_city' => NULL,
														'last_steeet' => NULL,
														'last_grandtotal' => NULL,
														'last_shipping' => NULL,
														'last_sales_tax' => NULL,
														'last_sales_tax_full_response' => NULL,
													)
												),
									'cart_locked' => 0
								);
		}

		$this->session_items = json_decode($_SESSION['cart']['items'], true);
		$this->cart_id = $_SESSION['cart']['cart_id'];
		$this->checkout_fields = $_SESSION['cart']['checkout_fields'];
		$this->delivery_fields = $_SESSION['cart']['delivery'];
		$this->paypal_order_id = $_SESSION['cart']['paypal_order_id'];
		$this->synchrony_client_token = $_SESSION['cart']['synchrony_client_token'];
		$this->synchrony_verification_response = $_SESSION['cart']['synchrony_verification_response'];
		$this->coupon_id = $_SESSION['cart']['coupon_id'];
		$this->is_cart_mode = $is_cart_mode;
		$this->tax_api = $_SESSION['cart']['tax_api'];
		$this->cart_locked = $_SESSION['cart']['cart_locked'];
		$this->cc_gateway = $cc_gateway;
	}

	public function getDatabaseHandle() {
		return $this->conndb;
	}

	/* check whether cart mode or quote mode is activated */ 
	public function isCartModeEnabled() {
		$query = "SELECT `quote-site` FROM tbl_sitedetails";
		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);

		if($result->num_rows == 0) {
			$cart_mode = 1;
		}
		else {
			$row = $result->fetch_assoc();
			if($row['quote-site'] == 1)
				$cart_mode = 1;
			else
				$cart_mode = 0;
		}

		return $cart_mode;
	}

	/* check whether current mode is cart mode or not */
	public function isCartMode() {
		return $this->is_cart_mode;
	}

	/* check whether tax api to be used for tax calculation */
	public function hasTaxApi() {
		return $this->tax_api['enabled'];
	}

	/* check whether cart is locked for any updates to added items */
	public function isCartLocked() {
		return $this->cart_locked;
	}

	/* lock cart for any updates */
	public function lockCart() {
		$this->cart_locked = 1;
		$_SESSION['cart']['cart_locked'] = 1;
	}

	/* unlock cart to enable to update cart items */
	public function unlockCart() {
		$this->cart_locked = 0;
		$_SESSION['cart']['cart_locked'] = 0;
	}

	/* get the current credit cart api to be used */
	public function getCreditCardGateway() {
		return $this->cc_gateway;
	}

	/* get delivery details */
	public function getDeliveryDetails() {
		return array(
				'delivery_type' => $this->delivery_fields['delivery_type'],
				'has_delivery_with_install' => $this->delivery_fields['has_delivery_with_install'],
			);
	}

	/* get current delivery type set */
	public function getDeliveryType() {
		return $this->delivery_fields['delivery_type'];
	}

	/* check whether Delivery + Installation type is allowed */
	public function hasDeliveryWithInstall() {
		return $this->delivery_fields['has_delivery_with_install'];
	}

	/* validate whether delivery type chosen is allowed or not */
	public function checkDeliveryType($delivery_type) {
		$allowed = array('SHIP', 'PICKUP');

		if($this->hasDeliveryWithInstall() == 1)
			$allowed[] = 'SHIP-INSTALL';
			
		if(!in_array($delivery_type, $allowed))
			throw new Exception('Error : Incorrect delivery type');
	}

	/* set delivery type */
	public function setDeliveryType($delivery_type) {
		$this->checkDeliveryType($delivery_type);

		$this->delivery_fields['delivery_type'] = $delivery_type;
		$_SESSION['cart']['delivery']['delivery_type'] = $delivery_type;
	}

	/* get titles of all delivery types */
	public function getDeliveryTitles() {
		return $this->delivery_titles;
	}

	/* get current zip code */
	public function getZipCode() {
		return $this->delivery_fields['delivery_zip'];
	}

	/* set zip code & delivery type */
	protected function setZipCode($zip_code, $delivery_type) {
		$this->delivery_fields['delivery_zip'] = $zip_code;
		$_SESSION['cart']['delivery']['delivery_zip'] = $zip_code;
		setcookie('shipping_zip', $zip_code, time() + (86400 * 30), "/");

		if($delivery_type != NULL)
			$this->setDeliveryType($delivery_type);
	}

	/* get current pickup store id */
	public function getPickupStore() {
		return $this->delivery_fields['pickup_store'];
	}

	/* set pickup store id & delivery type */
	public function setPickupStore($pickup_store_id) {
		$this->delivery_fields['pickup_store'] = $pickup_store_id;
		$_SESSION['cart']['delivery']['pickup_store'] = $pickup_store_id;

		$this->setDeliveryType('PICKUP');
	}

	/* get cart id */
	public function getCartID() {
		return $this->cart_id;
	}

	/* get checkout fields */
	public function getCheckoutFields() {
		return $this->checkout_fields;
	}

	/* set billing address checkout fields */
	protected function setBillingCheckoutFields($billing_checkout_fields) {
		$this->checkout_fields['billing'] = $billing_checkout_fields;
		$_SESSION['cart']['checkout_fields']['billing'] = $billing_checkout_fields;
	}

	/* set shipping address checkout fields */
	protected function setShippingCheckoutFields($shipping_checkout_fields) {
		$this->checkout_fields['shipping'] = $shipping_checkout_fields;
		$_SESSION['cart']['checkout_fields']['shipping'] = $shipping_checkout_fields;
	}

	/* set payment checkout fields */
	protected function setPaymentCheckoutFields($payment_checkout_fields) {
		$this->checkout_fields['payment'] = $payment_checkout_fields;
		$_SESSION['cart']['checkout_fields']['payment'] = $payment_checkout_fields;
	}

	/* set paypal order id during paypal express checkout api */
	public function setPaypalOrderID($paypal_order_id) {
		$this->paypal_order_id = $paypal_order_id;
		$_SESSION['cart']['paypal_order_id'] = $paypal_order_id;
	}

	/* get paypal order id */
	public function getPaypalOrderID() {
		return $this->paypal_order_id;
	}

	/* get taxjar last api request data */
	public function getTaxApiLastRequest() {
		return $this->tax_api['params'];
	}

	/* set data for taxjar last api request */
	public function setTaxApiLastRequest($to_address, $grand_total, $shipping, $sales_tax, $sales_tax_api_response) {
		$this->tax_api['params']['last_zip'] = $to_address['to_zip'];
		$this->tax_api['params']['last_state'] = $to_address['to_state'];
		$this->tax_api['params']['last_city'] = $to_address['to_city'];
		$this->tax_api['params']['last_street'] = $to_address['to_street'];
		$this->tax_api['params']['last_grandtotal'] = $grand_total;
		$this->tax_api['params']['last_shipping'] = $shipping;
		$this->tax_api['params']['last_sales_tax'] = $sales_tax;
		$this->tax_api['params']['last_sales_tax_full_response'] = $sales_tax_api_response;

		$_SESSION['cart']['tax_api']['params']['last_zip'] = $to_address['to_zip'];
		$_SESSION['cart']['tax_api']['params']['last_state'] = $to_address['to_state'];
		$_SESSION['cart']['tax_api']['params']['last_city'] = $to_address['to_city'];
		$_SESSION['cart']['tax_api']['params']['last_street'] = $to_address['to_street'];
		$_SESSION['cart']['tax_api']['params']['last_grandtotal'] = $grand_total;
		$_SESSION['cart']['tax_api']['params']['last_shipping'] = $shipping;
		$_SESSION['cart']['tax_api']['params']['last_sales_tax'] = $sales_tax;
		$_SESSION['cart']['tax_api']['params']['last_sales_tax_full_response'] = $sales_tax_api_response;
	}

	/* check current coupon code details and save coupon id */
	public function checkCouponCode($coupon_code) {
		$coupon_details = $this->getCouponCodeDetails($coupon_code);
		if($coupon_details['invalid'] == 1)
			throw new Exception($coupon_details['invalid_reason']);

		$this->setCoupon($coupon_details['CouponID']);

		$coupon = array('coupon_code' => $coupon_code, 'coupon_code_text' => $coupon_details['CouponName']);
		return $coupon;
	}

	/* get details of coupon code from database */
	public function getCouponCodeDetails($coupon_code, $coupon_id = NULL, $customer_billing_email = NULL, $items = NULL) {
		$current_date = date('Y-m-d');
		$query = "SELECT * FROM tbl_coupons WHERE " . ($coupon_id != NULL ? "CouponID=" . $this->conndb->real_escape_string($coupon_id) : "CouponCode='" . $this->conndb->real_escape_string($coupon_code) . "'" ) . " AND CouponIsLive=1 AND CouponIsRemoved=0 AND CouponStartDate<=CURDATE() AND CouponEndDate>=CURDATE()";
		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);
		if($result->num_rows == 0) {
			return array('invalid' => 1, 'invalid_reason' => 'Invalid coupon code');
		}

		$coupon_details = $result->fetch_assoc();
		$coupon_min_purchase = $coupon_details['CouponMinimumPurchase'];
		$coupon_max_purchase = $coupon_details['CouponMaximumPurchase'];
		$single_use_coupon = $coupon_details['CouponIsSingleUse'];
		$coupon_id = $coupon_details['CouponID'];
		$brands_allowed = array();
		$categories_allowed = array();

		$cart_subtotal = $this->getCartSubtotalPrice();
		if($coupon_min_purchase != '0.00' && $cart_subtotal < floatval($coupon_min_purchase))  {
			return array('invalid' => 1, 'invalid_reason' => 'Minimum cart values should be $' . $coupon_min_purchase);
		}
		if($coupon_max_purchase != '0.00' && $cart_subtotal > floatval($coupon_max_purchase)) {
			return array('invalid' => 1, 'invalid_reason' => 'Coupon valid only for a maximum cart subtotal of $' . $coupon_max_purchase);
		}

		if($single_use_coupon == 1) {
			$client_id = $this->getClientID();
			if($client_id == -1 && $customer_billing_email != NULL) {
				$query = "SELECT id FROM tbl_clients WHERE email='" . $this->conndb->real_escape_string($customer_billing_email) . "'";
				$result = $this->conndb->query($query);
				if(!$result)
					throw new Exception($this->conndb->error);
				if($result->num_rows > 0) {
					$row = $result->fetch_assoc();
					$client_id = $row['id'];
				}
				else {
					$client_id = -1;
				}
			}

			if($client_id != -1) {
				$query = "SELECT id FROM tbl_orders WHERE client=" . $client_id . " AND OrderCoupon=" . $coupon_id;
				$result = $this->conndb->query($query);
				if(!$result)
					throw new Exception($this->conndb->error);
				if($result->num_rows > 0) {
					return array('invalid' => 1, 'invalid_reason' => 'Coupon can be used only once');
				}
			}
		}

		$query = "SELECT * FROM tbl_couponproducts WHERE CouponID=" . $coupon_id;
		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);
		if($result->num_rows > 0) {
			for($i=0; $i<$result->num_rows; $i++) {
				$row = $result->fetch_assoc();
				if(!is_null($row['BrandID']) && !in_array($row['BrandID'], $brands_allowed))
					$brands_allowed[] = $row['BrandID'];
				if(!is_null($row['CategoryID']) && !in_array($row['CategoryID'], $categories_allowed))
					$categories_allowed[] = $row['CategoryID'];
			}

			if($items == NULL) {
				$items_data = $this->getCartItems();
				$items = $items_data['items'];
			}
			$item_valid_coupon = 0;
			for($i=0; $i<sizeof($items); $i++) {
				$item_valid_brand = 1;
				$item_valid_category = 1;

				if(sizeof($brands_allowed) > 0 && !in_array($items[$i]['info']['pkBrand'], $brands_allowed))
					$item_valid_brand = 0;

				if(sizeof($categories_allowed) > 0 && !in_array($items[$i]['info']['pkCategory'], $categories_allowed))
					$item_valid_category = 0;

				if($item_valid_brand == 1 && $item_valid_category == 1) {
					$item_valid_coupon = 1;
					break;
				}
			}
			if($item_valid_coupon == 0)
				return array('invalid' => 1, 'invalid_reason' => 'Coupon not valid on selected brands / categories');
		}

		$coupon_details['invalid'] = 0;
		$coupon_details['brands_allowed'] = $brands_allowed;
		$coupon_details['categories_allowed'] = $categories_allowed;

		return $coupon_details;		
	}

	/* remove coupon code from cart */
	public function removeCoupon() {
		$this->setCoupon(NULL);
	}

	/* get current coupon id */
	public function getCoupon() {
		return $this->coupon_id;
	}

	/* set coupon id */
	public function setCoupon($coupon_id) {
		$this->coupon_id = $coupon_id;
		$_SESSION['cart']['coupon_id'] = $coupon_id;
	}

	/* get cart items */
	public function getCartItems() {
		$items = $this->session_items;

		/* query product details again if flag set so that latest details are saved in session */
		if($this->use_cached_info != 1) {
			for($i=0; $i<sizeof($items); $i++) {
				if($items[$i]['type'] == 'product') {
					$product_details = $this->getProductDetails($items[$i]['id'], $items[$i]['is_outlet']);
					$items[$i]['info'] = $product_details;
				}
			}
		}

		return array('items' => $items, 'cart_id' => $this->getCartID());
	}

	/* get number of items in cart */
	public function getCartItemsCount() {
		$items = $this->session_items;

		return sizeof($items);
	}

	/* get list of cart items & prices */
	public function getCartItemListWithPrices() {
		if(sizeof($this->session_items) == 0) {
			return array('items' => array());
		}

		$cart_items = $this->session_items;

		$all_category_ids = array();
		for($i=0; $i<sizeof($cart_items); $i++) {
			if($cart_items[$i]['type'] == 'product') {
				$product_details = $this->getProductDetails($cart_items[$i]['id'], $cart_items[$i]['is_outlet']);
				if($product_details['price'] == '0.00' || $product_details['price'] == NULL) {
					$this->removeProduct($cart_items[$i]['item_id']);
					throw new Exception('Error : ' . $product_details['title'] . ' is out of stock');
				}
				
				$cart_items[$i]['info'] = $product_details;

				if(!in_array($product_details['pkCategory'], $all_category_ids)) {
					$all_category_ids[] = $product_details['pkCategory'];
				}
			}
		}

		$list = array();
		$cart_subtotal_price = 0.00;
		$cart_taxable_price = 0.00;
		for($i=0; $i<sizeof($cart_items); $i++) {
			if($cart_items[$i]['type'] == 'product') {
				$product_quantity = $cart_items[$i]['quantity'];

				$product_total_price_float = floatval($cart_items[$i]['info']['price'])*$cart_items[$i]['quantity'];
				$list_item = array(
								'name' => $cart_items[$i]['info']['brand'] . ' ' . $cart_items[$i]['info']['title'] . ' (' . $cart_items[$i]['info']['sku'] . ')',
								'image' => $cart_items[$i]['info']['image'],
								'type' => 'product',
								'quantity' => $cart_items[$i]['quantity'],
								'price' => $cart_items[$i]['info']['price'],
								'id' => $cart_items[$i]['id'],
								'sku' => $cart_items[$i]['info']['sku'],
								'total_price' => number_format($product_total_price_float, 2, '.', ''),
								'total_price_formatted' => money_format('$%i', $product_total_price_float)
							);
				$list[] = $list_item;
				$cart_subtotal_price += $product_total_price_float;
				$cart_taxable_price += $product_total_price_float;

				if($cart_items[$i]['warranty'] != NULL) {
					for($j=0; $j<sizeof($cart_items[$i]['info']['warranty']['options']); $j++) {
						if($cart_items[$i]['warranty'] == $cart_items[$i]['info']['warranty']['options'][$j]['id']) {
							$warranty_total_price_float = floatval($cart_items[$i]['info']['warranty']['options'][$j]['price'])*$product_quantity;
							$list_item = array(
											'name' => $cart_items[$i]['info']['warranty']['options'][$j]['description'],
											'image' => $cart_items[$i]['info']['warranty']['options'][$j]['image'],
											'type' => 'warranty',
											'quantity' => $product_quantity,
											'price' => $cart_items[$i]['info']['warranty']['options'][$j]['price'],
											'id' => $cart_items[$i]['info']['warranty']['options'][$j]['warranty_id'],
											'sku' => $cart_items[$i]['info']['warranty']['options'][$j]['code'],
											'total_price' => number_format($warranty_total_price_float, 2, '.', ''),
											'total_price_formatted' => money_format('$%i', $warranty_total_price_float)
										);
							$list[] = $list_item;
							$cart_subtotal_price += $warranty_total_price_float;

							break;
						}
					}
				}

				if(sizeof($cart_items[$i]['addons']) > 0) {
					for($j=0; $j<sizeof($cart_items[$i]['info']['addons']); $j++) {
						if(in_array($cart_items[$i]['info']['addons'][$j]['id'], $cart_items[$i]['addons'])) {
							$addon_total_price_float = floatval($cart_items[$i]['info']['addons'][$j]['price'])*$product_quantity;
							$list_item = array(
											'name' => $cart_items[$i]['info']['addons'][$j]['description'],
											'image' => $cart_items[$i]['info']['addons'][$j]['image'],
											'type' => 'addon',
											'quantity' => $product_quantity,
											'price' => $cart_items[$i]['info']['addons'][$j]['price'],
											'id' => $cart_items[$i]['info']['addons'][$j]['id'],
											'sku' => $cart_items[$i]['info']['addons'][$j]['sku'],
											'total_price' => number_format($addon_total_price_float, 2, '.', ''),
											'total_price_formatted' => money_format('$%i', $addon_total_price_float)
										);
							$list[] = $list_item;
							$cart_subtotal_price += $addon_total_price_float;

							if($cart_items[$i]['info']['addons'][$j]['taxable'] == 1) 
								$cart_taxable_price += $addon_total_price_float;
						}
					}
				}

				if(sizeof($cart_items[$i]['options']) > 0) {
					for($j=0; $j<sizeof($cart_items[$i]['info']['options']); $j++) {
						if(array_key_exists(('option-' . $cart_items[$i]['info']['options'][$j]['option_type_id']), $cart_items[$i]['options'])) {
							for($k=0; $k<sizeof($cart_items[$i]['info']['options'][$j]['options_list']); $k++) {
								if($cart_items[$i]['options']['option-' . $cart_items[$i]['info']['options'][$j]['option_type_id']] == $cart_items[$i]['info']['options'][$j]['options_list'][$k]['id']) {
									$option_total_price_float = floatval($cart_items[$i]['info']['options'][$j]['options_list'][$k]['price'])*$product_quantity;
									$list_item = array(
													'name' => $cart_items[$i]['info']['options'][$j]['options_list'][$k]['description'],
													'image' => $cart_items[$i]['info']['options'][$j]['options_list'][$k]['image'],
													'type' => 'option',
													'quantity' => $product_quantity,
													'price' => $cart_items[$i]['info']['options'][$j]['options_list'][$k]['price'],
													'id' => $cart_items[$i]['info']['options'][$j]['options_list'][$k]['id'],
													'sku' => $cart_items[$i]['info']['options'][$j]['options_list'][$k]['sku'],
													'total_price' => number_format($option_total_price_float, 2, '.', ''),
													'total_price_formatted' => money_format('$%i', $option_total_price_float)
												);
									$list[] = $list_item;
									$cart_subtotal_price += $option_total_price_float;

									$cart_taxable_price += $option_total_price_float;

									break;
								}
							}
						}
					}
				}
			}
		}

		$coupon_discount = 0.00;
		if($this->coupon_id != NULL) {
			$coupon_details = $this->getCouponCodeDetails(NULL, $this->coupon_id, NULL, $cart_items);
			if($coupon_details['invalid'] == 0) {
				$brands_allowed = $coupon_details['brands_allowed'];
				$categories_allowed = $coupon_details['categories_allowed'];
				if(sizeof($brands_allowed) == 0 && sizeof($categories_allowed) == 0) {
					$cart_discountable_subtotal = $cart_subtotal_price;
				}
				else {
					$cart_discountable_subtotal = 0.00;
					for($i=0; $i<sizeof($cart_items); $i++) {
						$item_valid_brand = 1;
						$item_valid_category = 1;

						if(sizeof($brands_allowed) > 0 && !in_array($cart_items[$i]['info']['pkBrand'], $brands_allowed))
							$item_valid_brand = 0;

						if(sizeof($categories_allowed) > 0 && !in_array($cart_items[$i]['info']['pkCategory'], $categories_allowed))
							$item_valid_category = 0;

						if($item_valid_brand == 1 && $item_valid_category == 1) {
							$product_quantity = $cart_items[$i]['quantity'];
							$product_total_price_float = floatval($cart_items[$i]['info']['price'])*$cart_items[$i]['quantity'];
							$cart_discountable_subtotal += $product_total_price_float;

							if($cart_items[$i]['warranty'] != NULL) {
								for($j=0; $j<sizeof($cart_items[$i]['info']['warranty']['options']); $j++) {
									if($cart_items[$i]['warranty'] == $cart_items[$i]['info']['warranty']['options'][$j]['id']) {
										$warranty_total_price_float = floatval($cart_items[$i]['info']['warranty']['options'][$j]['price'])*$product_quantity;
										$cart_discountable_subtotal += $warranty_total_price_float;

										break;
									}
								}
							}

							if(sizeof($cart_items[$i]['addons']) > 0) {
								for($j=0; $j<sizeof($cart_items[$i]['info']['addons']); $j++) {
									if(in_array($cart_items[$i]['info']['addons'][$j]['id'], $cart_items[$i]['addons'])) {
										$addon_total_price_float = floatval($cart_items[$i]['info']['addons'][$j]['price'])*$product_quantity;
										$cart_discountable_subtotal += $addon_total_price_float;
									}
								}
							}

							if(sizeof($cart_items[$i]['options']) > 0) {
								for($j=0; $j<sizeof($cart_items[$i]['info']['options']); $j++) {
									if(array_key_exists(('option-' . $cart_items[$i]['info']['options'][$j]['option_type_id']), $cart_items[$i]['options'])) {
										for($k=0; $k<sizeof($cart_items[$i]['info']['options'][$j]['options_list']); $k++) {
											if($cart_items[$i]['options']['option-' . $cart_items[$i]['info']['options'][$j]['option_type_id']] == $cart_items[$i]['info']['options'][$j]['options_list'][$k]['id']) {
												$option_total_price_float = floatval($cart_items[$i]['info']['options'][$j]['options_list'][$k]['price'])*$product_quantity;
												$cart_discountable_subtotal += $option_total_price_float;

												break;
											}
										}
									}
								}
							}
						}
					}
				}

				if($coupon_details['CouponIsPercentage'] == 1)
					$coupon_discount = (floatval($coupon_details['CouponAmount'])/100)*number_format($cart_discountable_subtotal, 2, '.', '');
				else 
					$coupon_discount = floatval($coupon_details['CouponAmount']);

				if($coupon_details['CouponMaximumAmt'] != '0.00' && $coupon_discount > floatval($coupon_details['CouponMaximumAmt']))
					$coupon_discount = floatval($coupon_details['CouponMaximumAmt']);

				$cart_subtotal_price = $cart_subtotal_price - $coupon_discount;
			}
		}
		$cart_discount_price = number_format($coupon_discount, 2, '.', '');
		$cart_discount = array('price' => $cart_discount_price, 'formatted_price' => money_format('$%i', $cart_discount_price));
		if($coupon_discount != 0) {
			$cart_discount['coupon_code'] = $coupon_details['CouponCode'];
			$cart_discount['coupon_code_text'] = $coupon_details['CouponName'];
			$cart_discount['coupon_id'] = $this->coupon_id;
		}

		$cart_subtotal_price = number_format($cart_subtotal_price, 2, '.', '');
		$cart_subtotal = array('price' => $cart_subtotal_price, 'formatted_price' => money_format('$%i', $cart_subtotal_price));

		$cart_shipping_price = number_format($this->getCartShippingPrice($all_category_ids, $cart_subtotal_price), 2, '.', '');
		$cart_shipping = array('price' => $cart_shipping_price, 'formatted_price' => money_format('$%i', $cart_shipping_price));

		if($this->hasTaxApi() == 1) {
			$cart_tax_api = $this->getCartTaxPriceViaApi($cart_taxable_price, $cart_shipping_price);
			if($cart_tax_api == 'NA') {
				$cart_tax_price = 'NA';
				$cart_tax = array('price' => $cart_tax_price, 'formatted_price' => $cart_tax_price);

				$cart_total_price = number_format(($cart_subtotal_price + $cart_shipping_price), 2, '.', '');
				$cart_total = array('price' => $cart_total_price, 'formatted_price' => money_format('$%i', $cart_total_price));
			}
			else {
				$cart_tax_price = number_format($cart_tax_api, 2, '.', '');
				$cart_tax = array('price' => $cart_tax_price, 'formatted_price' => money_format('$%i', $cart_tax_price));
				
				$cart_total_price = number_format(($cart_subtotal_price + $cart_shipping_price + $cart_tax_price), 2, '.', '');
				$cart_total = array('price' => $cart_total_price, 'formatted_price' => money_format('$%i', $cart_total_price));
			}
		}
		else {
			$cart_tax_price = number_format($this->getCartTaxPrice($cart_taxable_price), 2, '.', '');
			$cart_tax = array('price' => $cart_tax_price, 'formatted_price' => money_format('$%i', $cart_tax_price));

			$cart_total_price = number_format(($cart_subtotal_price + $cart_shipping_price + $cart_tax_price), 2, '.', '');
			$cart_total = array('price' => $cart_total_price, 'formatted_price' => money_format('$%i', $cart_total_price));
		}

		$data = array(
					'items' => $list, 
					'subtotal_price' => $cart_subtotal,
					'shipping_price' => $cart_shipping,
					'tax_price' => $cart_tax,
					'total_price' => $cart_total,
					'discount_price' => $cart_discount
				);

		return $data;
	}

	/* validate billing address checkout fields provided by user */
	public function validateBillingCheckoutFields($billing_input) {
		$us_states_list_allowed = array('AL', 'AK', 'AZ', 'AR', 'CA', 'CO', 'CT', 'DE', 'DC', 'FL', 'GA', 'HI', 'ID', 'IL', 'IN', 'IA', 'KS', 'KY', 'LA', 'ME', 'MD', 'MA', 'MI', 'MN', 'MS', 'MO', 'MT', 'NE', 'NV', 'NH', 'NJ', 'NM', 'NY', 'NC', 'ND', 'OH', 'OK', 'OR', 'PA', 'RI', 'SC', 'SD', 'TN', 'TX', 'UT', 'VT', 'VA', 'WA', 'WV', 'WI', 'WY');
		$countries_allowed = array('US');

		/* billing validation */
		if(!isset($billing_input['name']) || strlen(trim($billing_input['name'])) == 0)
			throw new Exception('Error : Billing name not in correct format');

		if(!isset($billing_input['email']) || filter_var($billing_input['email'], FILTER_VALIDATE_EMAIL) === false)
			throw new Exception('Error : Billing email is incorrect');

		if(!isset($billing_input['phone']) || filter_var($billing_input['phone'], FILTER_VALIDATE_INT) === false || strlen(trim($billing_input['phone'])) != 10)
			throw new Exception('Error : Billing phone is incorrect');

		if(!isset($billing_input['street_1']) || strlen(trim($billing_input['street_1'])) == 0)
			throw new Exception('Error : Billing street address is not in correct format');

		if(!isset($billing_input['city']) || strlen(trim($billing_input['city'])) == 0)
			throw new Exception('Error : Billing city is not in correct format');

		if(!isset($billing_input['state']) || !in_array($billing_input['state'], $us_states_list_allowed))
			throw new Exception('Error : Billing state is incorrect');

		if(!isset($billing_input['zip']) || filter_var($billing_input['zip'], FILTER_VALIDATE_INT) === false || strlen(trim($billing_input['zip'])) != 5)
			throw new Exception('Error : Billing zip is incorrect');

		if(!isset($billing_input['country']) || !in_array($billing_input['country'], $countries_allowed))
			throw new Exception('Error : Billing country is incorrect');

		$billing = array(
						'name' => trim($billing_input['name']),
						'email' => trim($billing_input['email']),
						'phone' => trim($billing_input['phone']),
						'street_1' => trim($billing_input['street_1']),
						'street_2' => trim($billing_input['street_2']),
						'city' => trim($billing_input['city']),
						'state' => trim($billing_input['state']),
						'zip' => trim($billing_input['zip']),
						'country' => trim($billing_input['country'])
					);
		$name_parts = explode(' ', $billing['name']);
		if(sizeof($name_parts) == 1) {
			$billing['last_name'] = '';
			$billing['first_name'] = $billing['name'];
		}
		else {
			$billing['last_name'] = array_pop($name_parts);
			$billing['first_name'] = implode(' ', $name_parts);
		}

		$this->setBillingCheckoutFields($billing);
	}

	/* validate shipping address checkout fields provided by user */
	public function validateShippingCheckoutFields($shipping_input) {
		$us_states_list_allowed = array('AL', 'AK', 'AZ', 'AR', 'CA', 'CO', 'CT', 'DE', 'DC', 'FL', 'GA', 'HI', 'ID', 'IL', 'IN', 'IA', 'KS', 'KY', 'LA', 'ME', 'MD', 'MA', 'MI', 'MN', 'MS', 'MO', 'MT', 'NE', 'NV', 'NH', 'NJ', 'NM', 'NY', 'NC', 'ND', 'OH', 'OK', 'OR', 'PA', 'RI', 'SC', 'SD', 'TN', 'TX', 'UT', 'VT', 'VA', 'WA', 'WV', 'WI', 'WY');
		$countries_allowed = array('US');
		$delivery_type = $this->getDeliveryType();

		/* delivery validation */
		$this->checkDeliveryType($delivery_type);

		/* shipping validation */
		if($delivery_type == 'PICKUP') {
			$this->checkPickupStore($shipping_input['pickup_store']);
		}
		else {
			if(!isset($shipping_input['name']) || strlen(trim($shipping_input['name'])) == 0)
				throw new Exception('Error : Shipping name not in correct format');

			if(!isset($shipping_input['email']) || filter_var($shipping_input['email'], FILTER_VALIDATE_EMAIL) === false)
				throw new Exception('Error : Shipping email is incorrect');

			if(!isset($shipping_input['phone']) || filter_var($shipping_input['phone'], FILTER_VALIDATE_INT) === false || strlen(trim($shipping_input['phone'])) != 10)
				throw new Exception('Error : Shipping phone is incorrect');

			if(!isset($shipping_input['street_1']) || strlen(trim($shipping_input['street_1'])) == 0)
				throw new Exception('Error : Shipping street address is not in correct format');

			if(!isset($shipping_input['city']) || strlen(trim($shipping_input['city'])) == 0)
				throw new Exception('Error : Shipping city is not in correct format');

			if(!isset($shipping_input['state']) || !in_array($shipping_input['state'], $us_states_list_allowed))
				throw new Exception('Error : Shipping state is incorrect');

			if(!isset($shipping_input['zip']) || filter_var($shipping_input['zip'], FILTER_VALIDATE_INT) === false || strlen(trim($shipping_input['zip'])) != 5)
				throw new Exception('Error : Shipping zip is incorrect');

			if(!isset($shipping_input['country']) || !in_array($shipping_input['country'], $countries_allowed))
				throw new Exception('Error : Shipping country is incorrect');

			$this->checkZipCode($shipping_input['zip'], $shipping_input['delivery_type']);
			$this->checkShippingZipStateMatch($shipping_input['zip'], $shipping_input['state']);
		}

		if($delivery_type == 'PICKUP') {
			$shipping = array(
							'delivery_type' => $delivery_type,
							'pickup_store' => trim($shipping_input['pickup_store']),
							'name' => NULL,
							'email' => NULL,
							'phone' => NULL,
							'street_1' => NULL,
							'street_2' => NULL,
							'city' => NULL,
							'state' => NULL,
							'zip' => NULL,
							'country' => NULL,
						);
		}
		else {
			$shipping = array(
							'delivery_type' => $delivery_type,
							'pickup_store' => NULL,
							'name' => trim($shipping_input['name']),
							'email' => trim($shipping_input['email']),
							'phone' => trim($shipping_input['phone']),
							'street_1' => trim($shipping_input['street_1']),
							'street_2' => trim($shipping_input['street_2']),
							'city' => trim($shipping_input['city']),
							'state' => trim($shipping_input['state']),
							'zip' => trim($shipping_input['zip']),
							'country' => trim($shipping_input['country'])
						);
			$name_parts = explode(' ', $shipping['name']);
			if(sizeof($name_parts) == 1) {
				$shipping['last_name'] = '';
				$shipping['first_name'] = $shipping['name'];
			}
			else {
				$shipping['last_name'] = array_pop($name_parts);
				$shipping['first_name'] = implode(' ', $name_parts);
			}
		}

		$this->setShippingCheckoutFields($shipping);
	}

	/* validate payment checkout fields provided by user */
	public function validatePaymentCheckoutFields($payment_input) {
		$credit_card_year_allowed = array('2020', '2021', '2022', '2023', '2024', '2025', '2026');
		$credit_card_month_allowed = array('01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12');
		$credit_card_length_allowed = array(13, 15, 16);
		$payment_options_allowed = array('CC', 'PAYPAL', 'SYNCHRONY', 'WELLSFARGO', 'AMAZONPAY', 'QUOTE');
		$cc_gateways_allowed = array('AUTHORIZE', 'PAYPAL-PAYMENTS-PRO', 'PAYPAL-PAYFLOW');
		$cc_gateway = $this->getCreditCardGateway();

		/* payment validation */
		if(!isset($payment_input['type']) || !in_array($payment_input['type'], $payment_options_allowed))
			throw new Exception('Error : Payment type is incorrect');

		if($payment_input['type'] == 'QUOTE') {
			if($this->isCartMode() != 0)
				throw new Exception('Error : Cart is not in Quote mode');
		}
		else if($payment_input['type'] == 'CC') {
			if(!isset($payment_input['card_number']) || filter_var($payment_input['card_number'], FILTER_VALIDATE_INT) === false || !in_array(strlen(trim($payment_input['card_number'])), $credit_card_length_allowed))
				throw new Exception('Error : Credit card number is not in correct format');

			if(!isset($payment_input['card_name']) || strlen(trim($payment_input['card_name'])) == 0)
				throw new Exception('Error : Credit card name not in correct format');

			if(!isset($payment_input['card_ccv']) || !preg_match('/^[0-9]{1,}$/', $payment_input['card_ccv']))
				throw new Exception('Error : Credit card CCV is not in correct format');

			if(!isset($payment_input['card_month']) || !in_array($payment_input['card_month'], $credit_card_month_allowed))
				throw new Exception('Error : Credit card expiry month is incorrect');

			if(!isset($payment_input['card_year']) || !in_array($payment_input['card_year'], $credit_card_year_allowed))
				throw new Exception('Error : Credit card expiry year is incorrect');

			if(!in_array($cc_gateway, $cc_gateways_allowed))
				throw new Exception('Error : Credit card gateway type is incorrect');
		}

		$payment = array(
						'type' => $payment_input['type'],
						'card_number' => NULL,  
						'card_name' => NULL, 
						'card_ccv' => NULL, 
						'card_month' => NULL, 
						'card_year' => NULL,
						'cc_gateway' => NULL
					);
		if($payment['type'] == 'CC') {
			$payment['card_number'] = trim($payment_input['card_number']);
			$payment['card_name'] = trim($payment_input['card_name']);
			$payment['card_ccv'] = trim($payment_input['card_ccv']);
			$payment['card_month'] = trim($payment_input['card_month']);
			$payment['card_year'] = trim($payment_input['card_year']);
			$payment['cc_gateway'] = $cc_gateway;
		}

		$this->setPaymentCheckoutFields($payment);
	}

	/* get product, warranty & addons details from database */ 
	protected function getProductDetails($product_id, $is_outlet = 0) {
		$ProductDetails = new ProductDetails($this->conndb, NULL, NULL, NULL, NULL, NULL, $product_id, $is_outlet);
		$product_details = $ProductDetails->getProductBasicDetails();
		$warranty_details = $ProductDetails->getProductWarrantyDetails();
		$addons_details = $ProductDetails->getProductAddons();
		$options_details = $ProductDetails->getProductOptions();

		$product = $this->formatProductInfoForShoppingCart($product_details, $warranty_details, $addons_details, $options_details);

		return $product;
	}

	/* format product details from database to be saved in session as required */
	public function formatProductInfoForShoppingCart($product_details, $warranty_details, $addons_details, $options_details) {
		$product = array(
						'title' => $product_details['ModelDescription'], 
						'brand' => $product_details['CommonBrandName'], 
						'image' => $product_details['thumb'], 
						'price' => $product_details['Sale'], 
						'discount' => $product_details['discount'], 
						'model' => $product_details['SKU'], 
						'sku' => $product_details['SKU'],
						'pkCategory' => $product_details['pkCategory'],
						'pkBrand' => $product_details['pkBrand'],
						'url' => $product_details['url'],
						'free_shipping' => $product_details['free_shipping']
					);

		$warranty_formatted = array('options' => array());
		foreach($warranty_details as $id => $details) {
			$warranty_formatted['options'][] = array(
												'name' => $details['name'], 
												'price' => $details['price'], 
												'id' => $id, 
												'description' => $details['name']. ' Extended Warranty for ' . $product['brand'] . ' ' . $product['sku'], 
												'code' => $details['war_code'],
												'warranty_id' => $details['id'],
												'image' => '/_img/ele/warranty_thumb.jpg',
												'tooltip' => $details['tooltip']
											);
		}
		$product['warranty'] = $warranty_formatted;
		
		for($i=0; $i<sizeof($addons_details); $i++) {
			$addons_details[$i]['description'] = $addons_details[$i]['description'] . ' for ' . $product['brand'] . ' ' . $product['sku'];
			$addons_details[$i]['image'] = '/_img/ele/no_image_available_50px.jpg';
		}
		$product['addons'] = $addons_details;

		for($i=0; $i<sizeof($options_details); $i++) {
			for($j=0; $j<sizeof($options_details[$i]['options_list']); $j++) {
				$options_details[$i]['options_list'][$j]['description'] = $options_details[$i]['options_list'][$j]['title'] . ' OPTION for ' . $product['brand'] . ' ' . $product['pos_sku'];
				$options_details[$i]['options_list'][$j]['image'] = '/_img/ele/no_image_available_50px.jpg';
			}
		}
		$product['options'] = $options_details;

		return $product;
	}

	/* add product to cart */
	public function addProduct($product_id, $warranty_id, $addons, $options, $info, $is_outlet) {
		if($this->isCartLocked())
			throw new Exception('Error : Cart locked for updates');

		$item_id = $this->checkExactProductIDPresent($product_id, $is_outlet);
		if($item_id != -1) {
			$this->changeProductWarranty($item_id, $warranty_id);
			$this->addOneProductQuantity($item_id);
			return;
		}

		$item_id = uniqid();
		$new_product = array('item_id' => $item_id, 'type' => 'product', 'id' => $product_id, 'is_outlet' => $is_outlet, 'quantity' => 1, 'info' => $info);
		$new_product['addons'] = $addons;
		if($warranty_id !== NULL)
			$new_product['warranty'] = $warranty_id;
		else
			$new_product['warranty'] = NULL;
		$new_product['options'] = $options;

		$this->session_items[] = $new_product;

		$this->updateSessionPrice();
	}

	/* change warranty of a particular cart item */
	public function changeProductWarranty($item_id, $warranty_id) {
		if($this->isCartLocked())
			throw new Exception('Error : Cart locked for updates');

		$index = $this->checkItemPresent($item_id);
		if($index == -1) 
			throw new Exception('Error : No item present');

		$old_warranty = $this->session_items[$index]['warranty'];

		if($old_warranty == $warranty_id)
			return;
			
		$this->session_items[$index]['warranty'] = $warranty_id;

		$this->updateSessionPrice();
	}

	/* change addons of a particular cart item */
	public function updateProductAddon($item_id, $addon_id, $update_type) {
		if($this->isCartLocked())
			throw new Exception('Error : Cart locked for updates');

		$index = $this->checkItemPresent($item_id);
		if($index == -1) 
			throw new Exception('Error : No item present');

		if($update_type == 'ADD') {
			if(in_array($addon_id, $this->session_items[$index]['addons']))
				return;

			$this->session_items[$index]['addons'][] = $addon_id;
		}
		else if($update_type == 'REMOVE') {
			$addon_index = array_search($addon_id, $this->session_items[$index]['addons']);
			unset($this->session_items[$index]['addons'][$addon_index]);
			$this->session_items[$index]['addons'] = array_values($this->session_items[$index]['addons']);
		}

		$this->updateSessionPrice();
	}

	/* change options of a particular cart item */
	public function changeProductOption($item_id, $option_type_id, $options_list_id) {
		$index = $this->checkItemPresent($item_id);
		if($index == -1) 
			throw new Exception('Error : No item present');

		if($options_list_id == 0) {
			if(array_key_exists('option-' . $option_type_id, $this->session_items[$index]['options'])) {
				unset($this->session_items[$index]['options']['option-' . $option_type_id]);
			}
		}
		else {
			$this->session_items[$index]['options']['option-' . $option_type_id] = $options_list_id;
		}

		$this->updateSessionPrice();
	}

	/* remove specific cart item */
	public function removeProduct($item_id) {
		if($this->isCartLocked())
			throw new Exception('Error : Cart locked for updates');

		$index = $this->checkItemPresent($item_id);
		if($index == -1) 
			throw new Exception('Error : No item present');

		unset($this->session_items[$index]);
		$this->session_items = array_values($this->session_items);

		$this->updateSessionPrice();
	}

	/* add one quantity of a specific cart item */
	public function addOneProductQuantity($item_id) {
		if($this->isCartLocked())
			throw new Exception('Error : Cart locked for updates');

		$index = $this->checkItemPresent($item_id);
		if($index == -1) 
			throw new Exception('Error : No item present');

		$this->session_items[$index]['quantity']++;

		$this->updateSessionPrice();

		return $this->session_items[$index]['quantity'];
	}

	/* remove one quantity of a specific cart item */
	public function removeOneProductQuantity($item_id) {
		if($this->isCartLocked())
			throw new Exception('Error : Cart locked for updates');

		$index = $this->checkItemPresent($item_id);
		if($index == -1) 
			throw new Exception('Error : No item present');

		if($this->session_items[$index]['quantity'] == 1)
			$this->removeProduct($item_id);
		else
			$this->session_items[$index]['quantity']--;

		$this->updateSessionPrice();

		return $this->session_items[$index]['quantity'];
	}

	/* check whether the specific item is already present in cart */
	protected function checkItemPresent($item_id) {
		$index = -1;
		for($i=0; $i<sizeof($this->session_items); $i++) {
			if($this->session_items[$i]['item_id'] == $item_id)
				$index = $i;
		}

		return $index;
	}

	/* check whether the same product is already present in cart, including outlet item or not */
	protected function checkExactProductIDPresent($product_id, $is_outlet) {
		$item_id = -1;
		for($i=0; $i<sizeof($this->session_items); $i++) {
			if($this->session_items[$i]['id'] == $product_id && $this->session_items[$i]['type'] == 'product' && $this->session_items[$i]['is_outlet'] == $is_outlet) {
				$item_id = $this->session_items[$i]['item_id'];
				break;
			}
		}

		return $item_id;
	}
	
	/* check whether the same product is already present in cart */
	protected function checkExactProductPresent($product_id, $warranty_id) {
		$item_id = -1;
		for($i=0; $i<sizeof($this->session_items); $i++) {
			if($this->session_items[$i]['id'] == $product_id && $this->session_items[$i]['type'] == 'product') {
				if($this->session_items[$i]['warranty'] == $warranty_id) {
					$item_id = $this->session_items[$i]['item_id'];
					break;
				}
			}
		}

		return $item_id;
	}

	/* update items from temp cart table */
	protected function updateTempCart() {
		$items = $this->session_items;

		/* start transaction */
		$this->conndb->autocommit(FALSE);

		/* delete old rows */
		$query = "DELETE FROM tbl_temp_cart_details WHERE uniqueID=" . $this->conndb->real_escape_string($this->cart_id);
		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);
		
		/* insert all rows */
		if(sizeof($items) > 0) {
			$insert_rows = array();
			for($i=0; $i<sizeof($items); $i++) {
				if($items[$i]['type'] == 'product') {
					$insert_rows[] = "(" . 
								$this->conndb->real_escape_string($this->cart_id) . ", '" . 
								$this->conndb->real_escape_string($items[$i]['info']['price']) . "', " . 
								$this->conndb->real_escape_string($items[$i]['id']) . ", '" . 
								$this->conndb->real_escape_string($items[$i]['info']['brand'] . ' ' . $items[$i]['info']['title']) . "', " . 
								$this->conndb->real_escape_string($items[$i]['quantity']) . ", '" . 
								$this->conndb->real_escape_string($items[$i]['info']['sku']) . "'" . 
							")";

					if($items[$i]['warranty'] != NULL) {
						for($j=0; $j<sizeof($items[$i]['info']['warranty']['options']); $j++) {
							if($items[$i]['warranty'] == $items[$i]['info']['warranty']['options'][$j]['id']) {
								$insert_rows[] = "(" . 
											$this->conndb->real_escape_string($this->cart_id) . ", '" . 
											$this->conndb->real_escape_string($items[$i]['info']['warranty']['options'][$j]['price']) . "', " . 
											$this->conndb->real_escape_string($items[$i]['info']['warranty']['options'][$j]['warranty_id']) . ", '" . 
											$this->conndb->real_escape_string($items[$i]['info']['warranty']['options'][$j]['description']) . "', " . 
											$this->conndb->real_escape_string($items[$i]['quantity']) . ", '" . 
											$this->conndb->real_escape_string($items[$i]['info']['warranty']['options'][$j]['code']) . "'" . 
										")";
								break;
							}
						}
					}
				}
			}
			$query = "INSERT INTO tbl_temp_cart_details (uniqueID,originalPrice,itemID,itemName,itemQuantity,itemSKU) VALUES " . implode(',', $insert_rows);
			$result = $this->conndb->query($query);
			if(!$result)
				throw new Exception($this->conndb->error);
		}

		/* end transaction */
		$this->conndb->commit();
	}

	/* update cart price */
	protected function updateSessionPrice() {
		$this->updateTempCart();
		$_SESSION['cart']['items'] = json_encode($this->session_items);
		$_SESSION['cart']['subtotal'] = $this->getCartSubtotalPrice();
	}

	/* get cart subtotal from session variables */
	public function getCartSubtotalPrice() {
		$cart_items = $this->session_items;
		$cart_subtotal_price = 0.00;
		for($i=0; $i<sizeof($cart_items); $i++) {
			if($cart_items[$i]['type'] == 'product') {
				$product_quantity = $cart_items[$i]['quantity'];

				$product_total_price_float = floatval($cart_items[$i]['info']['price'])*$cart_items[$i]['quantity'];
				$cart_subtotal_price += $product_total_price_float;
				
				if($cart_items[$i]['warranty'] != NULL) {
					for($j=0; $j<sizeof($cart_items[$i]['info']['warranty']['options']); $j++) {
						if($cart_items[$i]['warranty'] == $cart_items[$i]['info']['warranty']['options'][$j]['id']) {
							$warranty_total_price_float = floatval($cart_items[$i]['info']['warranty']['options'][$j]['price'])*$product_quantity;
							$cart_subtotal_price += $warranty_total_price_float;

							break;
						}
					}
				}

				if(sizeof($cart_items[$i]['addons']) > 0) {
					for($j=0; $j<sizeof($cart_items[$i]['info']['addons']); $j++) {
						if(in_array($cart_items[$i]['info']['addons'][$j]['id'], $cart_items[$i]['addons'])) {
							$addon_total_price_float = floatval($cart_items[$i]['info']['addons'][$j]['price'])*$product_quantity;
							$cart_subtotal_price += $addon_total_price_float;

							break;
						}
					}
				}
			}
		}

		$cart_subtotal_price = number_format($cart_subtotal_price, 2, '.', '');

		return $cart_subtotal_price;
	}

	/* add an order to database */
	public function placeOrder($billing, $shipping, $payment, $cart_prices, $transaction_details) {
		/* start transaction */
		$this->conndb->autocommit(FALSE);

		/* check if the client exists, if not insert a new one */
		$query = "SELECT id FROM tbl_clients WHERE email='" . $this->conndb->real_escape_string($billing['email']) . "'";
		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);
		if($result->num_rows == 0) {
			$query = "INSERT INTO tbl_clients (email,password,fname,lname,address1,address2,city,region,countries,zip,phone,tstampinsert,dateinsert) VALUES ('" . 
						$this->conndb->real_escape_string($billing['email']) . "','" .
						$this->conndb->real_escape_string(md5(uniqid())) . "','" .
						$this->conndb->real_escape_string($billing['first_name']) . "','" .
						$this->conndb->real_escape_string($billing['last_name']) . "','" .
						$this->conndb->real_escape_string($billing['street_1']) . "','" .
						$this->conndb->real_escape_string($billing['street_2']) . "','" .	
						$this->conndb->real_escape_string($billing['city']) . "','" .
						$this->conndb->real_escape_string($billing['state']) . "','" .
						$this->conndb->real_escape_string($billing['country']) . "','" .
						$this->conndb->real_escape_string($billing['zip']) . "','" .
						$this->conndb->real_escape_string($billing['phone']) . "'," .
						time() . ",'" .
						date('Y-m-d') . "'" .
					")";
			$result = $this->conndb->query($query);
			if(!$result)
				throw new Exception($this->conndb->error);
			$client_id = $this->conndb->insert_id;
		}
		else {
			$row = $result->fetch_assoc();
			$client_id = $row['id'];

			$query = "UPDATE tbl_clients SET " . 
						"fname='" . $this->conndb->real_escape_string($billing['first_name']) . "', " .
						"lname='" . $this->conndb->real_escape_string($billing['last_name']) . "', " .
						"address1='" . $this->conndb->real_escape_string($billing['street_1']) . "', " .
						"address2='" . $this->conndb->real_escape_string($billing['street_2']) . "', " .
						"city='" . $this->conndb->real_escape_string($billing['city']) . "', " .
						"region='" . $this->conndb->real_escape_string($billing['state']) . "', " .
						"countries='" . $this->conndb->real_escape_string($billing['country']) . "', " .
						"zip='" . $this->conndb->real_escape_string($billing['zip']) . "', " .
						"phone='" . $this->conndb->real_escape_string($billing['phone']) . "'" .
					" WHERE id=" . $client_id;
			$result = $this->conndb->query($query);
			if(!$result)
				throw new Exception($this->conndb->error);
		}

		if($this->hasTaxApi() == 1) {
			$taxjar_last_request = $this->getTaxApiLastRequest();
			$taxjar_last_sales_tax_full_response = $taxjar_last_request['last_sales_tax_full_response'];
		}

		/* insert order */
		$query = "INSERT INTO tbl_orders (client,subtotal,shipping,tax,grandtotal,orderDate,tstampinsert,shipfname,shiplname,shipaddress1,shipaddress2,shipcity,shipregion,shipcountries,shipzip,shipemail,shipphone,transactionid,authorizationcode,fullResponse,localpickup,localpickuplocation,discount,discountDescription,OrderCoupon,paymenttype" . ($this->hasTaxApi() == 1 ? ",tax_api_response" : "") . ") VALUES (" . 
					$this->conndb->real_escape_string($client_id) . ",'" .
					$this->conndb->real_escape_string($cart_prices['subtotal_price']['price']) . "','" .
					$this->conndb->real_escape_string($cart_prices['shipping_price']['price']) . "','" .
					$this->conndb->real_escape_string($cart_prices['tax_price']['price']) . "','" .
					$this->conndb->real_escape_string($cart_prices['total_price']['price']) . "','" .
					date('Y-m-d') . "', " .
					time() . ", " .
					($shipping['delivery_type'] == 'PICKUP' ? "NULL" : ("'" . $this->conndb->real_escape_string($shipping['first_name']) . "'")) . "," .
					($shipping['delivery_type'] == 'PICKUP' ? "NULL" : ("'" . $this->conndb->real_escape_string($shipping['last_name']) . "'")) . "," .
					($shipping['delivery_type'] == 'PICKUP' ? "NULL" : ("'" . $this->conndb->real_escape_string($shipping['street_1']) . "'")) . "," .
					($shipping['delivery_type'] == 'PICKUP' ? "NULL" : ("'" . $this->conndb->real_escape_string($shipping['street_2']) . "'")) . "," .
					($shipping['delivery_type'] == 'PICKUP' ? "NULL" : ("'" . $this->conndb->real_escape_string($shipping['city']) . "'")) . "," .
					($shipping['delivery_type'] == 'PICKUP' ? "NULL" : ("'" . $this->conndb->real_escape_string($shipping['state']) . "'")) . "," .
					($shipping['delivery_type'] == 'PICKUP' ? "NULL" : ("'" . $this->conndb->real_escape_string($shipping['country']) . "'")) . "," .
					($shipping['delivery_type'] == 'PICKUP' ? "NULL" : ("'" . $this->conndb->real_escape_string($shipping['zip']) . "'")) . "," .
					($shipping['delivery_type'] == 'PICKUP' ? "NULL" : ("'" . $this->conndb->real_escape_string($shipping['email']) . "'")) . "," .
					($shipping['delivery_type'] == 'PICKUP' ? "NULL" : ("'" . $this->conndb->real_escape_string($shipping['phone']) . "'")) . "," .
					($transaction_details['transId'] == NULL ? "NULL" : ("'" . $this->conndb->real_escape_string($transaction_details['transId']) . "'")) . "," .
					($transaction_details['authCode'] == NULL ? "NULL" : ("'" . $this->conndb->real_escape_string($transaction_details['authCode']) . "'")) . "," .
					($transaction_details['full_response'] == NULL ? "NULL" : ("'" . $this->conndb->real_escape_string($transaction_details['full_response']) . "'")) . "," .
					($shipping['delivery_type'] == 'PICKUP' ? "1" : "0") . "," .
					($shipping['delivery_type'] == 'PICKUP' ? $this->conndb->real_escape_string($shipping['pickup_store']) : "NULL") . "," .
					($cart_prices['discount_price']['price'] == 0 ? "NULL" : ("'" . $this->conndb->real_escape_string($cart_prices['discount_price']['price']) . "'")) . "," .
					($cart_prices['discount_price']['price'] == 0 ? "NULL" : ("'" . $this->conndb->real_escape_string($cart_prices['discount_price']['coupon_code_text']) . "'")) . "," .
					($cart_prices['discount_price']['price'] == 0 ? "-1" : $this->conndb->real_escape_string($cart_prices['discount_price']['coupon_id'])) . ",'" .
					$payment['type'] . "'" .
					($this->hasTaxApi() == 1 ? (",'" . $this->conndb->real_escape_string($taxjar_last_sales_tax_full_response) . "'") : "") .
				")";
		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);
		$order_id = $this->conndb->insert_id;

		/* insert order items */
		$items = $cart_prices['items'];
		for($i=0; $i<sizeof($items); $i++) {
			$query = "INSERT INTO tbl_orderdetails (orders,productID,sku,details,qty,price,tstampinsert,dateinsert,moredetails) VALUES (" . 
						$order_id . "," .
						$this->conndb->real_escape_string($items[$i]['id']) . ",'" .
						$this->conndb->real_escape_string($items[$i]['sku']) . "','" .
						$this->conndb->real_escape_string($items[$i]['name']) . "'," .
						$this->conndb->real_escape_string($items[$i]['quantity']) . ",'" .
						$this->conndb->real_escape_string($items[$i]['price']) . "'," .
						time() . ",'" .
						date('Y-m-d') . "', '" .
						$this->conndb->real_escape_string($items[$i]['type']) . "'" .
					")";
			$result = $this->conndb->query($query);
			if(!$result)
				throw new Exception($this->conndb->error);
		}

		/* delete from temp cart */
		$query = "DELETE FROM tbl_temp_cart_details WHERE uniqueID=" . $this->conndb->real_escape_string($this->cart_id);
		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);

		/* end transaction */
		$this->conndb->commit();

		$this->clearCart();

		return $order_id;
	}

	/* clear the current cart */
	public function clearCart() {
		$this->session_items = array();
		$this->cart_id = NULL;
		$this->shipping_zip = NULL;
		$this->checkout_fields = NULL;
		$this->delivery_fields = NULL;
		$this->paypal_order_id = NULL;
		$this->synchrony_verification_response = NULL;

		unset($_SESSION['cart']);
	}

	/* get authorize.net api keys */ 
	public function getAuthorizeApiKeys() {
		/* for sandbox testing purposes. uncomment below query to get live details from database */ 
		return array('authorize-test-acount' => '1', 'authorize-gateway-login' => '99QFkxN3T', 'authorize-gateway-password' => '39pA2A8KE8de968J');

		/*$query = "SELECT `authorize-test-acount`,`authorize-gateway-login`,`authorize-gateway-password` FROM tbl_sitedetails";
		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);

		if($result->num_rows == 0)
			throw new Exception('Error: No Authorize.net API keys present');

		$row = $result->fetch_assoc();
		return $row;*/
	}

	/* get paypal express checkout api keys */
	public function getPaypalApiKeys() {
		return array('paypal-test-acount' => '1', 'paypal-client-id' => 'Aa2qu1mHDd-vgoOiGpPm74yIqVHUv9BARJHzWX6aqUc_LVm82__VprfiBz8zELYiF4I9xv1vtF57jEdO', 'paypal-client-secret' => 'EHAQm8M3eglEZ3l0BqtzC1DMJOJ8rTPw_VcNphWev-rUusdU2L-piBlmsraHpN8zmfAFGPdWL4GWvlRH');
	}

	/* get synchrony api keys */
	public function getSynchronyApiKeys() {
		return array('synchrony-test-acount' => '1', 'synchrony-merchant-number' => '5348120770709186', 'synchrony-merchant-password' => 'e09aoqnf6yMlLEUz/rzpOQ==');
	}

	/* set token from synchrony api */
	public function setSynchronyClientToken($synchrony_client_token) {
		$this->synchrony_client_token = $synchrony_client_token;
		$_SESSION['cart']['synchrony_client_token'] = $synchrony_client_token;
	}

	/* get token from synchrony api */
	public function getSynchronyClientToken() {
		return $this->synchrony_client_token;
	}

	/* set details from synchrony api response */
	public function setSynchronyVerificationResponse($synchrony_response) {
		$this->synchrony_verification_response = $synchrony_response;
		$_SESSION['cart']['synchrony_verification_response'] = $synchrony_response;

		$this->checkout_fields['billing']['name'] = $synchrony_response['FirstName'] . ' ' . $synchrony_response['LastName'];
		$_SESSION['cart']['checkout_fields']['billing']['name'] = $synchrony_response['FirstName'] . ' ' . $synchrony_response['LastName'];
		
		$this->checkout_fields['billing']['first_name'] = $synchrony_response['FirstName'];
		$_SESSION['cart']['checkout_fields']['billing']['first_name'] = $synchrony_response['FirstName'];

		$this->checkout_fields['billing']['last_name'] = $synchrony_response['LastName'];
		$_SESSION['cart']['checkout_fields']['billing']['last_name'] = $synchrony_response['LastName'];

		$this->checkout_fields['billing']['street_1'] = $synchrony_response['Address1'];
		$_SESSION['cart']['checkout_fields']['billing']['street_1'] = $synchrony_response['Address1'];

		$this->checkout_fields['billing']['city'] = $synchrony_response['City'];
		$_SESSION['cart']['checkout_fields']['billing']['city'] = $synchrony_response['City'];

		$this->checkout_fields['billing']['state'] = $synchrony_response['State'];
		$_SESSION['cart']['checkout_fields']['billing']['state'] = $synchrony_response['State'];

		$this->checkout_fields['billing']['zip'] = $synchrony_response['ZipCode'];
		$_SESSION['cart']['checkout_fields']['billing']['zip'] = $synchrony_response['ZipCode'];

		$this->checkout_fields['payment']['type'] = 'SYNCHRONY';
		$_SESSION['cart']['checkout_fields']['payment']['type'] = 'SYNCHRONY';
	}

	/* get details from synchrony api response */
	public function getSynchronyVerificationResponse() {
		return $this->synchrony_verification_response;
	}

	/* clear details from synchrony api response */
	public function clearSynchronyData() {
		$this->setSynchronyClientToken(NULL);
		$this->setSynchronyVerificationResponse(NULL);
	}

	/* get wells fargo api keys */
	public function getWellsFargoApiKeys() {
		$api_settings = array(
							'SANDBOX' => array('gateway_entity_id' => '2619638100-0c4e6-136bc', 'gateway_company_id' => '2619638100', 'consumer_key' => 'Wend5sfLCSiIeZJFIff6hJwobK0a', 'consumer_secret' => 'KffDxf2NFZeQjOeNa9IsB5XM6eEa', 'merchant_number' => '577442176021319'),
							'VALIDATION' => array('gateway_entity_id' => '2619638100-0c4e6-136bc', 'gateway_company_id' => '2619638100', 'consumer_key' => 'xtb9uLEAcTZ3dMJfcmM872BTDoAa', 'consumer_secret' => 'AuBvKV88ZlsBEOt9FsYxrSeAoEAa', 'merchant_number' => '577442176021319'),
						);
		$certificate = array('private_key_path' => '/usr/www/users/ekidtv/siteontime.com/development/wells-fargo/certificate/2619638100-tst.key', 'public_key_path' => '/usr/www/users/ekidtv/siteontime.com/development/wells-fargo/certificate/2619638100-0c4e6-136bc-tst.pem', 'password' => 'fghjkl');

		$wellsfargo_mode = 'VALIDATION';

		$wellsfargo_api_keys = array('mode' => $wellsfargo_mode);
		$wellsfargo_api_keys['gateway_entity_id'] = $api_settings[$wellsfargo_mode]['gateway_entity_id'];
		$wellsfargo_api_keys['gateway_company_id'] = $api_settings[$wellsfargo_mode]['gateway_company_id'];
		$wellsfargo_api_keys['consumer_key'] = $api_settings[$wellsfargo_mode]['consumer_key'];
		$wellsfargo_api_keys['consumer_secret'] = $api_settings[$wellsfargo_mode]['consumer_secret'];
		$wellsfargo_api_keys['merchant_number'] = $api_settings[$wellsfargo_mode]['merchant_number'];
		$wellsfargo_api_keys['certificate'] = $certificate;

		return $wellsfargo_api_keys;
	}

	/* update cart billing address as per wells fargi billing address */
	public function updateWellsFargoBillingAddress($account_details) {
		$checkout_fields = $this->getCheckoutFields();

		$wellsfargo_billing_address = array(
											'email' => $checkout_fields['billing']['email'],
											'phone' => $checkout_fields['billing']['phone'],
											'street_1' => $account_details['applicant']['address']['address_1'],
											'street_2' => $account_details['applicant']['address']['address_2'],
											'city' => $account_details['applicant']['address']['city'],
											'state' => $account_details['applicant']['address']['state'],
											'zip' => $account_details['applicant']['address']['postal_code'],
											'country' => 'US'
										);
		$name_parts = explode(',', $account_details['applicant']['name']);
		$wellsfargo_billing_address['last_name'] = $name_parts[0];
		$wellsfargo_billing_address['first_name'] = explode(' ', $name_parts[1]);
		$wellsfargo_billing_address['first_name'] = $wellsfargo_billing_address['first_name'][0];
		$wellsfargo_billing_address['name'] = $wellsfargo_billing_address['first_name'] . ' ' . $wellsfargo_billing_address['last_name'];

		$this->setBillingCheckoutFields($wellsfargo_billing_address);
	}

	/* get amazon pay api keys */
	public function getAmazonPayApiKeys() {
		return array('amazonpay-test-acount' => '1', 'mws-access-key-id' => 'AKIAJR77OE6KM3WTCUDA', 'mws-secret-key' => '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', 'mws-seller-id' => 'ANT4TED4WZW40', 'amazonpay-client-id' => 'amzn1.application-oa2-client.aeabc924674a41e7b4b0cc0b8431926f');
	}

	/* get paypal payments pro api keys */
	public function getPaypalPaymentsProApiKeys() {
		return array('paypal-pro-test-acount' => '1', 'nvp-username' => 'sb-43c1hb3078995_api1.business.example.com', 'nvp-password' => 'E6RBC6X5ASH6L7DW', 'nvp-signature' => 'AMYulsHHZOlfocHV4sLoX.4bVIFIAtJ1MUiQzH.Uy5kZj-CMD7Qtw.Gx');
	}

	/* get paypal payflow api keys */
	public function getPaypalPayflowApiKeys() {
		$query = "SELECT `payflo-pro`,`payflo-username`,`payflo-vendor`,`payflo-password` FROM tbl_sitedetails";
		$result = $this->conndb->query($query);
		if(!$result)
			throw new Exception($this->conndb->error);

		if($result->num_rows == 0)
			throw new Exception('Error: No Paypal Payflow API keys present');

		$row = $result->fetch_assoc();
		return $row;
	}
}

$delivery_titles = array('SHIP' => DELIVERY_TYPE_SHIP_TITLE, 'SHIP-INSTALL' => DELIVERY_TYPE_SHIP_INSTALL_TITLE, 'PICKUP' => DELIVERY_TYPE_PICKUP_TITLE);
$ShoppingCart = new SiteShoppingCart($hostname_connDB, $username_connDB, $password_connDB, $database_connDB, HAS_SHIPPING_WITH_INSTALL, $delivery_titles, HAS_TAX_API, CC_GATEWAY);

?>