<?php

class SynchronyApi
{
	private $is_sandbox;

	private $merchant_id;

	private $merchant_password;

	public function __construct($merchant_id, $merchant_password, $is_sandbox = 0) {
		$this->merchant_id = $merchant_id;
		$this->merchant_password = $merchant_password;
		$this->is_sandbox = $is_sandbox;
	}

	public function getClientToken() {
		if($this->is_sandbox == 1)
			$api_url = 'https://ubuy.syf.com/DigitalBuy/authentication.do';
		else
			$api_url = 'https://buy.syf.com/DigitalBuy/authentication.do';			
		
		$curl_post = 'merchantId=' . $this->merchant_id . '&password=' . $this->merchant_password;
		$ch = curl_init();		
		curl_setopt($ch, CURLOPT_URL, $api_url);		
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Accept: application/json'));
		curl_setopt($ch, CURLOPT_POST, 1);		
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $curl_post);	
		$data = json_decode(curl_exec($ch), true);
		$http_code = curl_getinfo($ch,CURLINFO_HTTP_CODE);		
		if($http_code != 200) 
			throw new Exception('Error : Failed to receieve client token');
			
		return $data['clientToken'];
	}

	public function getPaymentStatus($client_token) {
		if($this->is_sandbox == 1)
			$api_url = 'https://usvcs.syf.com/v1.0/status/inquiry';
		else
			$api_url = 'https://svcs.syf.com/v1.0/status/inquiry';			
		
		$curl_post = json_encode(array('merchantNumber' => $this->merchant_id, 'password' => $this->merchant_password, 'userToken' => $client_token));
		$ch = curl_init();		
		curl_setopt($ch, CURLOPT_URL, $api_url);		
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json', 'Content-Length: ' . strlen($curl_post)));
		curl_setopt($ch, CURLOPT_POST, 1);		
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $curl_post);
		curl_setopt($ch, CURLOPT_TIMEOUT, 3);
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 3);	
		$data = json_decode(curl_exec($ch), true);
		//echo '<pre>';print_r($data);echo '</pre>';
		$http_code = curl_getinfo($ch,CURLINFO_HTTP_CODE);	

		if($http_code != 200) 
			throw new Exception('Error : Failed to verify client token (1)');
		
		if($http_code == 200 && $data['responseCode'] == '400')
			throw new Exception('Error : ' . $data['errors'][0]);

		if($http_code == 200 && !in_array($data['StatusCode'], array('000', '002', '003')))
			throw new Exception('Error : ' . (array_key_exists('errors', $data) ? $data['errors'][0] : $data['StatusMessage']));
			
		if(array_key_exists('ZipCode', $data))
			$data['ZipCode'] = substr($data['ZipCode'], 0, 5);
		
		return $data;
	}
}

?>