<?php

class WellsFargoApi
{
	private $request_type;

	private $gateway_entity_id;

	private $gateway_company_id;

	private $consumer_key;

	private $consumer_secret;

	private $certificate;

	public function __construct($request_type, $gateway_entity_id, $gateway_company_id, $consumer_key, $consumer_secret, $certificate) {
		$this->request_type = $request_type;
		$this->gateway_entity_id = $gateway_entity_id;
		$this->gateway_company_id = $gateway_company_id;
		$this->consumer_key = $consumer_key;
		$this->consumer_secret = $consumer_secret;
		$this->certificate = $certificate;
	}

	public function getApiKey() {
		if($this->request_type == 'SANDBOX')
			$api_url = 'https://api-sandbox.wellsfargo.com/token';
		elseif($this->request_type == 'VALIDATION')
			$api_url = 'https://api-certification.wellsfargo.com/token';
		elseif($this->request_type == 'PRODUCTION')
			$api_url = 'https://api.wellsfargo.com/token';

		$params = array('grant_type' => 'client_credentials', 'scope' => 'PLCCA-SDK-logs PLCCA-Applications PLCCA-Payment-Calculations PLCCA-Account-Details PLCCA-Transactions-Term PLCCA-Transactions-Authorization AUTH-SessionToken');

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $api_url);	
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Authorization: Basic ' . base64_encode($this->consumer_key . ':' . $this->consumer_secret)));	
		curl_setopt($ch, CURLOPT_POST, 1);		
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));	
		$data = json_decode(curl_exec($ch), true);
		$http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		curl_close($ch);
		if($http_code != 200) 
			throw new Exception('Error : Failed to get api key');

		$api_key = $data['access_token'];

		return $api_key;
	}

	public function checkConnection($api_key) {
		if($this->request_type == 'SANDBOX')
			$api_url = 'https://api-sandbox.wellsfargo.com/utilities/v1/hello-wellsfargo';
		elseif($this->request_type == 'VALIDATION')
			$api_url = 'https://api-certification.wellsfargo.com/utilities/v1/hello-wellsfargo';
		elseif($this->request_type == 'PRODUCTION')
			$api_url = 'https://api.wellsfargo.com/utilities/v1/hello-wellsfargo';

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $api_url);	
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Authorization: Bearer '. $api_key, 'Content-Type: application/json', 'gateway-company-id: ' . $this->gateway_company_id, 'request-id: ' . uniqid()));	
		curl_setopt($ch, CURLOPT_VERBOSE, 1);
		curl_setopt($ch, CURLOPT_SSLKEY, $this->certificate['private_key_path']);
		curl_setopt($ch, CURLOPT_SSLCERT, $this->certificate['public_key_path']);
		curl_setopt($ch, CURLOPT_SSLCERTPASSWD, $this->certificate['password']);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
		$data = json_decode(curl_exec($ch), true);
		if($errno = curl_errno($ch)) {
		    $error_message = curl_strerror($errno);
		    throw new Exception('cURL error ' . $errno . ' - ' . $error_message);
		}
		$http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		curl_close($ch);	
		if($http_code != 200) 
			throw new Exception('Error : Failed to connect to Hello API');
	}

	public function getSessionToken($api_key) {
		if($this->request_type == 'SANDBOX')
			$api_url = 'https://api-sandbox.wellsfargo.com/auth/v1/session-tokens';
		elseif($this->request_type == 'VALIDATION')
			$api_url = 'https://api-certification.wellsfargo.com/auth/v1/session-tokens';
		elseif($this->request_type == 'PRODUCTION')
			$api_url = 'https://api-sandbox.wellsfargo.com/auth/v1/session-tokens';

		$params = array('device_fingerprint' => array('ip_address' => $_SERVER['REMOTE_ADDR'], 'user_agent' => $_SERVER['HTTP_USER_AGENT']));

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $api_url);	
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Authorization: Bearer '. $api_key, 'Content-Type: application/json', 'gateway-entity-id: ' . $this->gateway_entity_id, 'client-request-id: ' . uniqid()));	
		curl_setopt($ch, CURLOPT_POST, 1);		
		curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));	
		curl_setopt($ch, CURLOPT_VERBOSE, 1);
		if($this->request_type == 'VALIDATION' || $this->request_type == 'PRODUCTION') {
			curl_setopt($ch, CURLOPT_SSLKEY, $this->certificate['private_key_path']);
			curl_setopt($ch, CURLOPT_SSLCERT, $this->certificate['public_key_path']);
			curl_setopt($ch, CURLOPT_SSLCERTPASSWD, $this->certificate['password']);
		}
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
		$data = json_decode(curl_exec($ch), true);
		if($errno = curl_errno($ch)) {
		    $error_message = curl_strerror($errno);
		    throw new Exception('cURL error ' . $errno . ' - ' . $error_message);
		}
		$http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		curl_close($ch);	
		if($http_code != 200) 
			throw new Exception('Error : Failed to get session token');

		$session_token = $data['session_token'];

		if($this->request_type == 'SANDBOX')
			$gateway_base_url = 'https://wfgaccess-sandbox.wellsfargo.com';
		elseif($this->request_type == 'VALIDATION')
			$gateway_base_url = 'https://wfgaccess-certification.wellsfargo.com';
		elseif($this->request_type == 'PRODUCTION')
			$gateway_base_url = 'https://wfgaccess.wellsfargo.com';

		return array('session_token' => $session_token, 'gateway_base_url' => $gateway_base_url);
	}

	public function accountInquiry($api_key, $merchant_number, $first_name, $last_name, $last_4_account) {
		if($this->request_type == 'SANDBOX')
			$api_url = 'https://api-sandbox.wellsfargo.com/credit-cards/private-label/new-accounts/v2/details';
		elseif($this->request_type == 'VALIDATION')
			$api_url = 'https://api-certification.wellsfargo.com/credit-cards/private-label/new-accounts/v2/details';
		elseif($this->request_type == 'PRODUCTION')
			$api_url = 'https://api.wellsfargo.com/credit-cards/private-label/new-accounts/v2/details';

		$params = array('merchant_number' => $merchant_number, 'transaction_code' => 'C2', 'first_name' => $first_name, 'last_name' => $last_name, 'last_4_account_number' => $last_4_account);

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $api_url);	
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Authorization: Bearer '. $api_key, 'Content-Type: application/json', 'gateway-entity-id: ' . $this->gateway_entity_id, 'client-request-id: ' . uniqid()));	
		curl_setopt($ch, CURLOPT_POST, 1);		
		curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));	
		curl_setopt($ch, CURLOPT_VERBOSE, 1);
		curl_setopt($ch, CURLINFO_HEADER_OUT, 1);
		if($this->request_type == 'VALIDATION' || $this->request_type == 'PRODUCTION') {
			curl_setopt($ch, CURLOPT_SSLKEY, $this->certificate['private_key_path']);
			curl_setopt($ch, CURLOPT_SSLCERT, $this->certificate['public_key_path']);
			curl_setopt($ch, CURLOPT_SSLCERTPASSWD, $this->certificate['password']);
		}
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
		$data = json_decode(curl_exec($ch), true);
		if($errno = curl_errno($ch)) {
		    $error_message = curl_strerror($errno);
		    throw new Exception('cURL error ' . $errno . ' - ' . $error_message);
		}
		$http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		curl_close($ch);	
		if($http_code != 200) 
			throw new Exception('Error : Failed to get account details - ' . $data['errors'][0]['description']);
		else if($data['transaction_status'] != 'H0')
			throw new Exception('Error : Failed to get account details - ' . $data['message']);

		return $data;
	}

	public function authorizePayment($api_key, $merchant_number, $amount, $account_number, $plan_number) {
		if($this->request_type == 'SANDBOX')
			$api_url = 'https://api-sandbox.wellsfargo.com/credit-cards/private-label/new-accounts/v2/payment/transactions/authorization';
		elseif($this->request_type == 'VALIDATION')
			$api_url = 'https://api-certification.wellsfargo.com/credit-cards/private-label/new-accounts/v2/payment/transactions/authorization';
		elseif($this->request_type == 'PRODUCTION')
			$api_url = 'https://api.wellsfargo.com/credit-cards/private-label/new-accounts/v2/payment/transactions/authorization';

		$params = array('merchant_number' => $merchant_number, 'amount' => $amount, 'account_number' => $account_number, 'plan_number' => $plan_number, 'authorization_number' => '000000');

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $api_url);	
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Authorization: Bearer '. $api_key, 'Content-Type: application/json', 'gateway-entity-id: ' . $this->gateway_entity_id, 'client-request-id: ' . uniqid()));	
		curl_setopt($ch, CURLOPT_POST, 1);		
		curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));	
		curl_setopt($ch, CURLOPT_VERBOSE, 1);
		curl_setopt($ch, CURLINFO_HEADER_OUT, 1);
		if($this->request_type == 'VALIDATION' || $this->request_type == 'PRODUCTION') {
			curl_setopt($ch, CURLOPT_SSLKEY, $this->certificate['private_key_path']);
			curl_setopt($ch, CURLOPT_SSLCERT, $this->certificate['public_key_path']);
			curl_setopt($ch, CURLOPT_SSLCERTPASSWD, $this->certificate['password']);
		}
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
		$data = json_decode(curl_exec($ch), true);
		if($errno = curl_errno($ch)) {
		    $error_message = curl_strerror($errno);
		    throw new Exception('cURL error ' . $errno . ' - ' . $error_message);
		}
		$http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		curl_close($ch);	
		if($http_code != 200) 
			throw new Exception('Error : Failed to authorize payment - ' . $data['errors'][0]['description']);
		else if($data['transaction_status'] != 'A1')
			throw new Exception('Error : Failed to authorize payment - ' . $data['status_message']);

		return $data;
	}
}

?>